@@ -1,9 +1,9 @@
 /*
- * Copyright (c) 2003-2013
+ * Copyright (c) 2003-2016
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  *
- * $Id: acs.h 2646 2013-02-27 20:10:40Z brachman $
+ * $Id: acs.h 2913 2016-10-18 19:55:39Z brachman $
  */
 
 /*****************************************************************************
@@ -223,6 +223,7 @@ typedef enum {
   ACS_DENIAL_REASON_BY_SIMPLE_REDIRECT = 907,
   ACS_DENIAL_REASON_CREDENTIALS_LIMIT  = 908,
   ACS_DENIAL_REASON_INACTIVITY         = 909,
+  ACS_DENIAL_REASON_ADMIN_REQUIRED     = 910,
   ACS_DENIAL_REASON_UNKNOWN            = 998,
   ACS_DENIAL_REASON_DEFAULT            = 999,
   ACS_DENIAL_REASON_LAST_RULE          = 999
@@ -244,14 +245,19 @@ typedef struct Acs_environment {
   int is_dacsexpr;
 } Acs_environment;
 
+/*
+ * Context for url_pattern matching, updated as a given request (a URL) is
+ * matched against each candidate pattern.
+ */
 typedef struct Acl_url_match {
-  int nsegs;
-  int nsegs_matched;
-  int exact;
-  int delegated;
-  char *subordinate;
-  Acl_rule *acl;
-  Service *s;
+  int nsegs;			/* Total number of path segments in the request. */
+  int nsegs_matched;	/* Best number of path segments matched so far. */
+  int exact;			/* Non-zero if an exact match has been found. */
+  int delegated;		/* Non-zero if delegated matching is being done. */
+  char *subordinate;	/* Used for access token testing. */
+  Acl_rule *acl;		/* The matching ACL rule. */
+  Service *s;			/* The matching Service element within the ACL rule. */
+  int el_num;			/* The matching Service element index. */
 } Acl_url_match;
 
 typedef enum {
@@ -298,6 +304,9 @@ typedef struct Acs_result {
   char *errmsg;
 } Acs_result;
   
+/*
+ * The final result of expression evaluation.  See acs_expr().
+ */
 typedef enum Acs_expr_result {
   /* The code for False must be zero. */
   ACS_EXPR_FALSE         = 0,
@@ -308,9 +317,14 @@ typedef enum Acs_expr_result {
   /* All error codes must be negative. */
   ACS_EXPR_SYNTAX_ERROR  = -2,
   ACS_EXPR_EVAL_ERROR    = -3,
-  ACS_EXPR_LEXICAL_ERROR = -4
+  ACS_EXPR_LEXICAL_ERROR = -4,
+  ACS_EXPR_UNDEF         = -5		/* Not an error, but neither True nor False. */
 } Acs_expr_result;
 
+#define EXPR_DEFAULT_DIGEST_DIGEST_NAME		"SHA1"
+#define EXPR_DEFAULT_HMAC_DIGEST_NAME		"SHA1"
+#define EXPR_DEFAULT_PBKDF2_DIGEST_NAME		"SHA1"
+
 static inline MAYBE_UNUSED int
 acs_expr_error_occurred(Acs_expr_result st)
 {
@@ -403,6 +417,8 @@ typedef enum {
   T_B8          = 80,
   T_B2          = 81,
 
+  T_IO          = 82,
+
   T_UNDEF       = -1
 } Token;
 
@@ -421,6 +437,17 @@ typedef struct Arglist {
   struct Arglist *next;
 } Arglist;
 
+/*
+ * A T_BSTRING value will always be printed as a hexadecimal string if
+ * it has a format of FORMAT_HEX, otherwise it will be printed as text
+ * if that makes sense.  See expr.c:force_string().
+ * XXX this could be extended to force other output formats
+ */
+typedef enum {
+  FORMAT_HEX       = 0,
+  FORMAT_PRINTABLE = 1
+} Bstring_format;
+
 /* A T_BSTRING */
 typedef struct Bstring {
   void *data;
@@ -454,6 +481,7 @@ typedef struct Value {
   int is_alist_reference;
   char *varname;
   int is_quoted;
+  Bstring_format format;
   union {
 	char *strval;
 	long intval;
@@ -482,9 +510,13 @@ typedef struct Acs_deny_result {
   Expr_result *result;
 } Acs_deny_result;
 
+#define ACS_DEFAULT_APPROVAL_DIGEST_NAME	"SHA1"
+#define ACS_TOKEN_HMAC_DIGEST_NAME			"SHA1"
+
 /* An access token for authorization caching. */
 typedef struct Acs_token {
-  char *issued_by;
+  char *issued_by_federation;
+  char *issued_by_jurisdiction;
   char *url_path;
   char *identity;
   Credentials *credentials;
@@ -573,11 +605,13 @@ extern Acs_expr_result acs_expr_list_eval(Dsvec *dsv, Acs_expr_ns_arg *ns_args,
 extern Ds *acs_string_operand(char *str, Acs_environment *env);
 extern Acs_expr_ns_arg *var_ns_to_acs_ns(Var_ns *namespaces);
 
+extern int acs_match_abs_url(char *acl_url, Acs_environment *env,
+							 Uri_pattern **patp);
 extern int acs_match_url_segs(char *url_pattern, Dsvec *dsv_req, int *exact);
 
 extern int acs_init_argv_namespace(Acs_environment *env, int argc,
 								   char **argv);
-extern int acs_is_readonly_namespace(char *ns);
+extern int acs_is_readonly_namespace(Var_ns *ns, char *name);
 
 extern int acs_token_create(Credentials *selected,
 							Acs_result *result, char **cookie_buf);
@@ -632,6 +666,7 @@ extern int access_token_delete(char *key);
 extern Kwv *access_token_get(char *key);
 
 extern int is_valid_id_attr(Dsvec *dsv, char *id);
+extern int is_valid_url_pattern(char *url_pattern);
 
 extern Value *init_value(Value *v, Token token, ...);
 
