@@ -1,9 +1,9 @@
 /*
- * Copyright (c) 2003-2016
+ * Copyright (c) 2003-2018
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  *
- * $Id: auth.h 2925 2016-11-09 17:01:21Z brachman $
+ * $Id: auth.h 2993 2018-01-26 00:58:54Z brachman $
  */
 
 /*****************************************************************************
@@ -50,6 +50,10 @@
 
 #include "dacs_crypto.h"
 
+#ifdef ENABLE_RADIUS_AUTH
+#include <radlib.h>
+#endif
+
 /*
  * For Rijndael (AES), these three key sizes are valid:
  * 128 bits (16 bytes), 192 bits (24 bytes), and 256 bits (32 bytes)
@@ -399,6 +403,17 @@ typedef struct Url_auth {
   Dsvec *options;
 } Url_auth;
 
+/*****************************************************/
+
+/* Used by local_radius_authenticate. */
+
+enum {
+  RADIUS_DEFAULT_TIMEOUT_SECS   = 3,
+  RADIUS_DEFAULT_DEAD_TIME_SECS = 2,	/* XXX This may be unnecessary... */
+  RADIUS_DEFAULT_MAX_TRIES      = 3
+};
+
+
 /*****************************************************/
 
 /* Used by local_pam_authenticate. */
@@ -667,6 +682,7 @@ typedef enum {
   AUTH_MODULE_TOKEN    = 11,
   AUTH_MODULE_PAM      = 12,
   AUTH_MODULE_TGMA     = 13,
+  AUTH_MODULE_RADIUS   = 14,
   AUTH_MODULE_UNKNOWN  = -1
 } Auth_module_id;
 
@@ -1573,12 +1589,21 @@ extern int auth_token_update(Auth_token *token);
 
 
 /* Local authentication modules */
+
 #ifdef ENABLE_APACHE_AUTH
 extern int local_apache_auth(char *username, char *password, char *aux,
 							 Http_auth_authorization *aa,
 							 Apache_module module, Apache_dbm dbm, char *path);
 #endif
 
+#ifdef ENABLE_RADIUS_AUTH
+extern int local_radius_auth(char *username, char *password, char *aux,
+							 char *radius_server, int radius_port,
+							 char *radius_secret, char *radius_ok_msg,
+							 int radius_timeout, int radius_max_tries,
+							 int radius_dead_time);
+#endif
+
 #ifdef ENABLE_CAS_AUTH
 extern int local_cas_auth(char *username, char *password, char *aux,
 						  Cas_auth *cas_auth);
@@ -1612,7 +1637,8 @@ extern int local_ldap_auth_by_attr(Uri *admin_url, char *admin_password,
   
 #ifdef ENABLE_NTLM_AUTH
 extern int local_ntlm_auth(char *username, char *password, char *aux,
-						   int smb_port, char *smb_server, char *smb_domain);
+						   int smb_port, char *smb_server, char *smb_domain,
+						   char *smb_workgroup);
 #endif
 
 #ifdef ENABLE_PAM_AUTH
