@@ -1,9 +1,9 @@
 /*
- * Copyright (c) 2003-2016
+ * Copyright (c) 2003-2018
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  *
- * $Id: dacs_crypto.h 2912 2016-10-18 19:54:07Z brachman $
+ * $Id: dacs_crypto.h 2991 2018-01-26 00:53:22Z brachman $
  */
 
 #ifndef _DACS_CRYPTO_H_
@@ -14,7 +14,6 @@
 #include <openssl/hmac.h>
 #include <openssl/pem.h>
 #include <openssl/rand.h>
-#include <openssl/md5.h>
 
 #include "blake2.h"
 #include "argon2.h"
@@ -469,6 +468,8 @@ extern unsigned char *crypto_hmac(const char *digest_name,
 								  unsigned char *key, unsigned int klen,
 								  unsigned char *value, unsigned int vlen,
 								  unsigned char **hmac, unsigned int *hlen);
+extern int crypto_hmac_key_length(char *digest_name);
+extern int crypto_hmac_hmac_length(char *digest_name);
 
 /* Random material */
 extern int crypto_randomize_buffer(unsigned char *buf, unsigned int len);
@@ -514,8 +515,9 @@ extern unsigned char *crypto_hkdf(unsigned char *ikm, unsigned int ikmlen,
 
 /* Encryption/Decryption */
 extern void crypto_set_padding(int enable);
+extern int crypto_cipher_block_size(char *cipher_name, unsigned int *len);
 extern int crypto_cipher_key_length(char *cipher_name, unsigned int *len);
-extern int crypto_cipher_iv_length(char *cipher_name, unsigned int *len);
+extern int crypto_lookup_cipher_iv_length(char *cipher_name, unsigned int *len);
 extern unsigned char *crypto_encipher(Crypto_function func, char *cipher,
 									  unsigned char *key, unsigned char **ivp,
 									  PKI_session_keys *sk,
