@@ -1,9 +1,9 @@
 /*
- * Copyright (c) 2003-2014
+ * Copyright (c) 2003-2017
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  *
- * $Id: http.h 2746 2014-12-31 17:44:20Z brachman $
+ * $Id: http.h 2983 2018-01-23 22:49:23Z brachman $
  */
 
 /*****************************************************************************
@@ -173,10 +173,10 @@ extern int http_request(Http *h, char *url);
 extern void http_close(Http *h);
 extern int http_read_response_body(Http *h, Ds *ds);
 extern FILE *http_read_response_stream(Http *h);
-extern Http_version *http_parse_version(char *version_str);
+extern Http_version *http_parse_version(const char *version_str);
 
 extern char *http_method_to_string(Http_method method);
-extern Http_method http_string_to_method(char *m);
+extern Http_method http_string_to_method(const char *m);
 extern char *http_get_user_agent_string(void);
 extern char *http_set_user_agent_string(char *str);
 
