@@ -1,9 +1,9 @@
 /*
- * Copyright (c) 2003-2013
+ * Copyright (c) 2003-2016
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  *
- * $Id: local.h 2646 2013-02-27 20:10:40Z brachman $
+ * $Id: local.h 2922 2016-10-20 21:16:41Z brachman $
  */
 
 /*****************************************************************************
@@ -108,6 +108,10 @@
 #include "var.h"
 #include "log.h"
 
+#ifdef ENABLE_ADDONS
+#include "addons.h"
+#endif
+
 #ifndef EAGAIN
 #define EAGAIN  EWOULDBLOCK
 #endif
@@ -160,7 +164,7 @@ typedef unsigned short in_port_t;
 
 enum {
 #ifdef NGROUPS
-  MAX_UNIX_GROUPS = NNGROUPS
+  MAX_UNIX_GROUPS = NGROUPS
 #else
   MAX_UNIX_GROUPS = 20
 #endif
@@ -248,7 +252,7 @@ typedef enum {
   DACS_UTILITY_PROC       = 4,	 /* A subprocess, config is optional */
   DACS_ACS                = 5,   /* DACS ACS, invoked by a web server */
   DACS_STANDALONE         = 6,	 /* Like DACS_UTILITY, except no config */
-  DACS_STANDALONE_NOARGS  = 7,	 /* Like DACS_STANDALONE, except no args */
+  DACS_STANDALONE_NOARGS  = 7,	 /* Like DACS_STANDALONE, except no DACS args */
   DACS_STANDALONE_SERVICE = 8,	 /* Like DACS_WEB_SERVICE, except no config */
   DACS_SERVER             = 9,   /* A server process; e.g., started by inetd */
   DACS_UNKNOWN_APP        = 99   /* Undefined state */
@@ -274,8 +278,8 @@ typedef struct DACS_app_args {
 } DACS_app_args;
 
 /* This is the level in effect before the configuration file is read. */
-#ifndef LOG_STARTUP_LEVEL
-#define LOG_STARTUP_LEVEL		LOG_WARN_LEVEL
+#ifndef LOG_INITIAL_LEVEL
+#define LOG_INITIAL_LEVEL		LOG_WARN_LEVEL
 #endif
 
 /*
@@ -310,7 +314,9 @@ typedef enum {
   EMIT_FORMAT_PHP       = 7,
   EMIT_FORMAT_FILE      = 8,	/* Output is returned as a MIME type. */
   EMIT_FORMAT_TEXT      = 9,	/* Output is to tty. */
-  EMIT_FORMAT_JSON      = 10
+  EMIT_FORMAT_JSON      = 10,
+  EMIT_FORMAT_URI       = 11,
+  EMIT_FORMAT_URL       = 12
 } Emit_format;
 
 #ifndef EMIT_FORMAT_DEFAULT
@@ -329,16 +335,17 @@ typedef enum {
 
 /* XXX This is actually part HTTP and part HTML */
 typedef struct Html_header_conf {
-  int no_cache;
-  int html;
-  int html_body;
-  char *css;
-  char *bgcolor;
-  char *title;
-  char *redirect_url;
-  char *status_code;
-  char *status_reason;
-  Dsvec *headers;
+  int no_cache;				/* If non-zero, emit <meta> tags to avoid caching. */
+  int html;					/* Emit an <html> tag and content. */
+  int html_body;			/* If non-zero, emit a <body> tag. */
+  char *css;				/* <link> tag's optional href attribute's URL. */
+  char *bgcolor;			/* <body> tag's optional bgcolor attribute value. */
+  char *title;				/* <title> tag's optional content. */
+  char *redirect_url;		/* Optional HTTP Location header value. */
+  char *status_code;		/* Optional numeric HTTP Status code. */
+  char *status_reason;		/* Optional reason text for HTTP Status code. */
+  Dsvec *head_tags;			/* Optional lines of content for <head>. */
+  Dsvec *headers;			/* These are HTTP headers, not HTML tags. */
 } Html_header_conf;
 
 typedef struct Common_status {
@@ -361,6 +368,24 @@ extern Parse_attr_tab common_status_attr_tab[];
 #define DEFAULT_NAMESPACE	"Temp"
 #endif
 
+/*
+ * Default initial characters that indicate a comment in configuration files.
+ * Refer to the loadrc operator to vfs() in dacs.exprs(5).
+ */
+#ifndef DEFAULT_RC_CHARS
+#define DEFAULT_RC_CHARS	"#\n"
+#endif
+
+/*
+ * Default prompts for interactive dacsexpr(1).
+ */
+#ifndef DEFAULT_PROMPT
+#define DEFAULT_PROMPT		"> "
+#endif
+#ifndef DEFAULT_PROMPT2
+#define DEFAULT_PROMPT2		">> "
+#endif
+
 typedef struct Directory_list {
   Dsvec *dsv;
   int exclude_dot_files;
@@ -389,6 +414,7 @@ typedef struct Directory_list {
 #define ITEM_TYPE_INFOCARDS						"infocards"
 #define ITEM_TYPE_JURISDICTION_KEYS				"jurisdiction_keys"
 #define ITEM_TYPE_LOCAL_PAM_AUTHENTICATE		"locate_pam_authenticate"
+#define ITEM_TYPE_NEWPASSWDS					"newpasswds"
 #define ITEM_TYPE_NOTICES						"notices"
 #define ITEM_TYPE_PASSWDS						"passwds"
 #define ITEM_TYPE_REVOCATIONS					"revocations"
@@ -585,7 +611,7 @@ extern Kwv *dacs_kwv_args;
 
 extern unsigned long trace_level;
 extern unsigned long verbose_level;
-extern int quiet_flag;
+extern int dacs_quiet_flag;
 extern Kwv *kwv_dacsoptions;
 extern char *standard_command_line_usage;
 extern int dacs_init_allow_dups_default;
@@ -620,6 +646,11 @@ extern "C" {
   #define _POSIX_C_SOURCE     199309L
   #define _BSD_SOURCE
 */
+
+#endif
+
+#ifndef CLOCK_VIRTUAL
+#define  CLOCK_VIRTUAL		CLOCK_PROCESS_CPUTIME_ID
 #endif
 
 #ifndef HAVE_CLOCK_GETTIME
@@ -664,15 +695,17 @@ extern void text_html(FILE *fp, char *buf);
 extern void env_html(FILE *fp);
 extern void hex_html(FILE *fp, unsigned char *buf, unsigned int len,
 					 unsigned int ncolumns);
-extern char *html_encode(char *str);
+extern char *html_encode(const char *str);
 
 extern char *create_temp_filename(char *suffix);
 extern FILE *create_temp_file(char *filename);
-extern int set_lock(char *lockfile, unsigned int maxage_secs,
-					unsigned int *remaining_secs);
 extern int check_lock(char *lockfile, unsigned int maxage_secs,
 					  unsigned int *remaining_secs);
+extern int create_lock(char *lockfile);
 extern int remove_lock(char *lockfile);
+extern int set_lock(char *lockfile, unsigned int maxage_secs,
+					unsigned int *remaining_secs);
+
 extern int edit_file(char *filename);
 extern int show_license(int argc, char **argv, int, void *main_out);
 extern int load_from_store(char *store_name, char *path, char **buf,
@@ -832,6 +865,18 @@ extern int dacshttp_main(int argc, char **argv, int do_init, void *main_out);
 extern int rlink_main(int argc, char **argv, int do_init, void *main_out);
 extern int sslclient_main(int argc, char **argv, int do_init, void *main_out);
 
+/* Macros in dacs_config.h map these names to the true names. */
+extern void *real_malloc(size_t);
+extern void *real_calloc(size_t, size_t);
+extern void *real_valloc(size_t);
+extern void *real_realloc(void *ptr, size_t);
+extern void real_free(void *ptr);
+extern void real_cfree(void *ptr);
+
+#ifdef ENABLE_BARCODE
+extern Ds *barcode_generate_element(Ds *ods, char *id, char *text);
+#endif
+
 #ifdef __cplusplus
 }
 #endif
