@@ -1,9 +1,9 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2016
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  *
- * $Id: vfs.h 2594 2012-10-19 17:28:49Z brachman $
+ * $Id: vfs.h 2910 2016-10-18 19:46:38Z brachman $
  */
 
 /*****************************************************************************
@@ -72,7 +72,8 @@ typedef enum {
   VFS_SET_FIELD_SEP     = 1,
   VFS_SET_COOKIES       = 2,
   VFS_SET_LOCK          = 3,
-  VFS_GET_CONTAINER     = 4
+  VFS_GET_CONTAINER     = 4,
+  VFS_GET_FIELD_SEP     = 5
 } Vfs_control_op;
 
 typedef enum {
@@ -164,6 +165,7 @@ extern void vfs_init(void);
 extern void vfs_term(void *bogus);
 extern Vfs_directive *vfs_init_directive(Vfs_directive *ovd);
 extern Vfs_directive *vfs_lookup_item_type(char *item_type);
+extern Dsvec *vfs_list_item_types(Dsvec *odsv);
 extern Vfs_op vfs_lookup_op(char *opname);
 extern int vfs_enabled(char *store_name);
 extern Dsvec *vfs_enabled_list(void);
