@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2013
+ * Copyright (c) 2003-2016
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -50,10 +50,10 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2013\n\
+"Copyright (c) 2003-2016\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: aclcheck.c 2643 2013-02-27 19:59:24Z brachman $";
+  "$Id: aclcheck.c 2902 2016-07-05 14:45:35Z brachman $";
 #endif
 
 #include "dacs.h"
@@ -1206,7 +1206,7 @@ dacsacl_main(int argc, char **argv, int do_init, void *main_out)
 		fprintf(stderr, "Error converting \"%s\" ACLs\n", ITEM_TYPE_ACLS);
 		return(-1);
 	  }
-	  if (!quiet_flag)
+	  if (!dacs_quiet_flag)
 		fprintf(stderr, "Converted \"%s\": %d rules\n", ITEM_TYPE_ACLS, n);
 
 #ifdef NOTDEF
@@ -1219,7 +1219,7 @@ dacsacl_main(int argc, char **argv, int do_init, void *main_out)
 		fprintf(stderr, "Error converting \"%s\" ACLs\n", ITEM_TYPE_DACS_ACLS);
 		return(-1);
 	  }
-	  if (!quiet_flag)
+	  if (!dacs_quiet_flag)
 		fprintf(stderr, "Converted \"%s\": %d rules\n", ITEM_TYPE_DACS_ACLS, n);
 #endif
 
@@ -1432,7 +1432,7 @@ dacsacl_main(int argc, char **argv, int do_init, void *main_out)
 		return(-1);
 	  }
 
-	  if (!quiet_flag)
+	  if (!dacs_quiet_flag)
 		fprintf(stderr, "Built index for \"%s\": %d rule%s\n",
 				ITEM_TYPE_ACLS, n, (n == 1) ? "" : "s");
 	}
@@ -1443,7 +1443,7 @@ dacsacl_main(int argc, char **argv, int do_init, void *main_out)
 		return(-1);
 	  }
 
-	  if (!quiet_flag)
+	  if (!dacs_quiet_flag)
 		fprintf(stderr, "Built index for \"%s\": %d rule%s\n",
 				ITEM_TYPE_DACS_ACLS, n, (n == 1) ? "" : "s");
 	}
@@ -1466,7 +1466,7 @@ dacsacl_main(int argc, char **argv, int do_init, void *main_out)
 		log_msg((LOG_WARN_LEVEL, "No config for item_type \"%s\"",
 				 ITEM_TYPE_ACLS));
 	}
-	else if (!quiet_flag)
+	else if (!dacs_quiet_flag)
 	  log_msg((LOG_WARN_LEVEL, "Ignoring undefined item_type \"%s\"",
 			   ITEM_TYPE_ACLS));
 
@@ -1480,7 +1480,7 @@ dacsacl_main(int argc, char **argv, int do_init, void *main_out)
 		log_msg((LOG_WARN_LEVEL, "No config for item_type \"%s\"",
 				 ITEM_TYPE_DACS_ACLS));
 	}
-	else if (!quiet_flag)
+	else if (!dacs_quiet_flag)
 	  log_msg((LOG_WARN_LEVEL, "Ignoring undefined item_type \"%s\"",
 			   ITEM_TYPE_DACS_ACLS));
   }
@@ -1499,7 +1499,7 @@ dacsacl_main(int argc, char **argv, int do_init, void *main_out)
 		log_msg((LOG_WARN_LEVEL, "No config for item_type \"%s\"",
 				 ITEM_TYPE_ACLS));
 	}
-	else if (!quiet_flag)
+	else if (!dacs_quiet_flag)
 	  log_msg((LOG_WARN_LEVEL, "Ignoring undefined item_type \"%s\"",
 			   ITEM_TYPE_ACLS));
 
@@ -1514,7 +1514,7 @@ dacsacl_main(int argc, char **argv, int do_init, void *main_out)
 		log_msg((LOG_WARN_LEVEL, "No config for item_type \"%s\"",
 				 ITEM_TYPE_DACS_ACLS));
 	}
-	else if (!quiet_flag)
+	else if (!dacs_quiet_flag)
 	  log_msg((LOG_WARN_LEVEL, "Ignoring undefined item_type \"%s\"",
 			   ITEM_TYPE_DACS_ACLS));
   }
