@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2013
+ * Copyright (c) 2003-2016
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -43,10 +43,10 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2013\n\
+"Copyright (c) 2003-2016\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: acs.c 2646 2013-02-27 20:10:40Z brachman $";
+  "$Id: acs.c 2904 2016-07-05 14:52:32Z brachman $";
 #endif
 
 #include "dacs.h"
@@ -212,7 +212,8 @@ make_activity_cookie_name(void)
 {
   char *cn;
 
-  cn = ds_xprintf("DACS:%s::::ACTIVITY", conf_val(CONF_FEDERATION_NAME));
+  cn = make_cookie_name(NULL, DACS_COOKIE_APP_NAME,
+						conf_val(CONF_FEDERATION_NAME), NULL, NULL, "ACTIVITY");
 
   return(cn);
 }
@@ -325,12 +326,13 @@ acs_is_inactive(Credentials *selected, Acs_activity *a)
 int
 main(int argc, char **argv)
 {
-  int error_occurred, i, ncred, st, xargc;
+  int disable_compat_check, error_occurred, i, ncred, st, xargc;
   int emitted_activity_cookie, in_error_handler, emit_activity_cookie;
   unsigned int alimit, ncookies;
   char **xargv, *p, *proxy_auth, *proxied_uri, *request_uri;
   char *service_authorization, *service_method, *service_uri;
   char *content_type, *errmsg, *remote_addr, *remote_host, *truncated, *uri;
+  char *mod_auth_dacs_version;
   char *dacs_acs_str, *htype, *web_server_env;
   Acs_activity *activity;
   Acs_denial_reason denial_reason;
@@ -364,6 +366,7 @@ main(int argc, char **argv)
   emit_activity_cookie = 0;
   emitted_activity_cookie = 0;
   activity = NULL;
+  disable_compat_check = 0;
 
   acs_init_result(&result);
 
@@ -394,7 +397,8 @@ main(int argc, char **argv)
 	  log_msg((LOG_DEBUG_LEVEL, "  Arg%d: \"%s\"", i, xargv[i]));
 
 	if (errmsg != NULL)
-	  log_msg((LOG_NOTICE_LEVEL, "%s", errmsg));
+	  log_msg((error_occurred ? LOG_ERROR_LEVEL : LOG_NOTICE_LEVEL, "%s",
+			   errmsg));
 
 	if (dacs_service_uri != NULL)
 	  p = ds_xprintf(" for %s", dacs_service_uri);
@@ -441,6 +445,9 @@ main(int argc, char **argv)
 	case ACS_DENIAL_REASON_INACTIVITY:
 	  denial_msg = "Access denied, inactivity";
 	  break;
+	case ACS_DENIAL_REASON_ADMIN_REQUIRED:
+	  denial_msg = "Access denied, admin identity required";
+	  break;
 	default:
 	case ACS_DENIAL_REASON_UNKNOWN:
 	  denial_msg = "Access denied, internal error";
@@ -483,6 +490,19 @@ main(int argc, char **argv)
 		log_msg((LOG_ERROR_LEVEL, "Invalid ACS_FAIL directive: \"%s\"", p));
 	}
 
+	/*
+	 * Something very bad has happened.
+	 * Avoid the possibility of a redirect to a DACS-wrapped resource,
+	 * which might cause this to happen again.
+	 */
+	if (error_occurred || denial_reason == ACS_DENIAL_REASON_UNKNOWN) {
+	  log_msg((LOG_ALERT_LEVEL,
+			   "An internal error has occurred, handler has been suppressed"));
+	  log_msg((LOG_ALERT_LEVEL, "Operator assistance is required"));
+	  log_msg((LOG_ERROR_LEVEL, "Exiting"));
+	  exit(1);
+	}
+
 	/*
 	 * If an error handler has been configured as a full URL, then pass
 	 * back to Apache an error document string for it to use as a custom 403
@@ -820,6 +840,10 @@ main(int argc, char **argv)
 	  proxy_exec_flag = 0;
 #endif
 	}
+	else if (streq(argv[i], "-dcc")) {
+	  /* For testing/debugging only! */
+	  disable_compat_check = 1;
+	}
 	else if (streq(argv[i], "-test")) {
 	  if (uri_pattern_match_selftest() != 0) {
 		fprintf(stderr, "Failed!\n");
@@ -927,6 +951,41 @@ main(int argc, char **argv)
   }
 #endif
 
+  mod_auth_dacs_version
+	= kwv_lookup_value(kwv, "SERVICE_MOD_AUTH_DACS_VERSION");
+  if (mod_auth_dacs_version != NULL)
+	log_msg((LOG_INFO_LEVEL, "SERVICE_MOD_AUTH_DACS_VERSION=%s",
+			 mod_auth_dacs_version));
+  else
+	log_msg((LOG_INFO_LEVEL, "SERVICE_MOD_AUTH_DACS_VERSION is undefined"));
+  if (!disable_compat_check) {
+	/*
+	 * Make sure that the version of mod_auth_dacs that has been used by
+	 * Apache is compatible with this version of DACS.
+	 * This was introduced in DACS 1.4.31 to protect against build and
+	 * install errors.
+	 *
+	 * If the versions do not match, we must be careful how we proceed.
+	 * We cannot rely on what mod_auth_dacs is telling DACS, so the safest
+	 * thing to do is fail quickly and loudly so that the problem will be fixed.
+	 */
+	if (mod_auth_dacs_version == NULL
+		|| !streq(mod_auth_dacs_version, DACS_VERSION_RELEASE)) {
+	  errmsg
+		= ds_xprintf("mod_auth_dacs version %s is incompatible with DACS %s",
+					 (mod_auth_dacs_version == NULL)
+					 ? "(unknown)" : mod_auth_dacs_version,
+					 DACS_VERSION_RELEASE);
+
+	  denial_reason = ACS_DENIAL_REASON_UNKNOWN;
+	  error_occurred = 1;
+	  goto fail;
+	}
+  }
+  else
+	log_msg((LOG_INFO_LEVEL,
+			 "Compatibility check for mod_auth_dacs is disabled"));
+
   /*
    * It is the SERVICE_URI, r->uri as passed here by mod_auth_dacs, that
    * we will use when matching against access control rules.
@@ -1374,6 +1433,13 @@ main(int argc, char **argv)
 
   /* Access is granted. */
 
+  if (mod_auth_dacs_version != NULL) {
+	kwv_add(result.auth_kwv, "DACS_MOD_AUTH_DACS_VERSION",
+			mod_auth_dacs_version);
+	log_msg((LOG_TRACE_LEVEL, "DACS_MOD_AUTH_DACS_VERSION=%s",
+			 mod_auth_dacs_version));
+  }
+
   if (emit_activity_cookie) {
 	acs_emit_set_header("Set-Cookie: %s", make_activity_cookie());
 	log_msg((LOG_TRACE_LEVEL, "Emitting activity cookie"));
