@@ -9,7 +9,7 @@ static const char copyright[] =
 "Copyright (c) 2003-2012\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: auth_grid.c 2586 2012-03-15 16:21:40Z brachman $";
+  "$Id: auth_grid.c 2610 2012-10-19 18:02:36Z brachman $";
 #endif
 
 #include "dacs.h"
@@ -481,7 +481,7 @@ auth_grid_parse_challenge(char *challenge_str)
   Dsvec *cells, *challenge;
 
   challenge = dsvec_init(NULL, sizeof(Auth_grid_coord));
-  cells = strsplit_re(challenge_str, " *, *", 0, &errmsg);
+  cells = strsplit_re(challenge_str, " *, *", 0, 0, &errmsg);
 
   for (i = 0; i < dsvec_len(cells); i++) {
 	cell = dsvec_ptr(cells, i, char *);
