@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2016
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -85,10 +85,10 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2012\n\
+"Copyright (c) 2003-2016\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: auth_transfer.c 2586 2012-03-15 16:21:40Z brachman $";
+  "$Id: auth_transfer.c 2928 2016-11-16 21:56:47Z brachman $";
 #endif
 
 #include "dacs.h"
@@ -1276,7 +1276,13 @@ main(int argc, char **argv)
   }
 
   if (st == -1) {
+#ifdef NOTDEF
+	/*
+	 * A suitable error message should have already been emitted, so don't
+	 * produce another one here...
+	 */
 	do_fail("main", errmsg);
+#endif
 	goto fail;
   }
 
