@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2015
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -43,22 +43,22 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2012\n\
+"Copyright (c) 2003-2015\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: cgiparse.c 2586 2012-03-15 16:21:40Z brachman $";
+  "$Id: cgiparse.c 2833 2015-08-19 19:13:00Z brachman $";
 #endif
 
 #ifdef DSSLIB
 #include "dsslib.h"
 #else
+#include "local.h"
+
 #include <unistd.h>
 #include <ctype.h>
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
-
-#include "local.h"
 #endif
 
 static char *log_module_name = "cgiparse";
@@ -485,7 +485,9 @@ cgiparse(FILE *fp, char *query_string, Kwv *kwv, Mime **mp)
 		&& ((ct->content_type == MIME_TYPE_APPLICATION
 			 && strcaseeq(ct->subtype, "x-www-form-urlencoded"))
 			|| (ct->content_type == MIME_TYPE_MULTIPART
-				&& strcaseeq(ct->subtype, "form-data;")))) {
+				/* The semicolon may be absent. */
+				&& (strcaseeq(ct->subtype, "form-data;")
+					|| strcaseeq(ct->subtype, "form-data"))))) {
 	  if (mime_parse(mime, input, kwv) == -1) {
 		log_msg((LOG_ERROR_LEVEL, "MIME parse of entity-body failed"));
 		return(-1);
@@ -570,9 +572,13 @@ usage(void)
 {
 
   fprintf(stderr,
-		  "Usage: cgiparse [-text|-html|-one|-arg var|-sh|-targ var] [-d] [-qs query-string] [-copy file] [-n name file] [-enc none|url|mime|dacs] [-in file] [-nonewline]\n");
-  fprintf(stderr, "where \"-n name file\" writes decoded form-data parts\n");
-  fprintf(stderr, " with a name parameter having value <name> to <file>\n");
+		  "Usage: cgiparse [-text|-html|-one|-arg var|-sh|-targ var]\n");
+  fprintf(stderr,
+		  "  [-checkdup] [-d] [-nodups | -duperror] [-qs query-string] [-copy file]\n");
+  fprintf(stderr,
+		  "  [-n name file] [-enc none|url|mime|dacs] [-in file] [-nonewline]\n");
+  fprintf(stderr, "Where \"-n name file\" writes decoded form-data parts\n");
+  fprintf(stderr, "  with a name parameter having value <name> to <file>\n");
 
   if (debug_fp != NULL)
 	fclose(debug_fp);
@@ -598,8 +604,8 @@ decode_val(char *enc_val, Cgiparse_enc enc_type)
 
   switch (enc_type) {
   case CGIPARSE_ENC_URL:
-	if ((decoded = (unsigned char *) url_decode(enc_val, &non_printable, &ds))
-		== NULL)
+	decoded = (unsigned char *) url_decode(enc_val, &non_printable, &ds);
+	if (decoded == NULL)
 	  return(NULL);
 	break;
 
@@ -650,22 +656,50 @@ shell_assign(char *var_name, Ds *var_value)
   return(assn);
 }
 
+static void
+emit_html(FILE *ofp, Kwv *kwv)
+{
+  FILE *fp;
+  Kwv_iter *iter;
+  Kwv_pair *pair;
+
+  if (kwv == NULL)
+    return;
+
+  kwv->error_msg = NULL;
+  if (ofp != NULL)
+	fp = ofp;
+  else
+    fp = stderr;
+
+  fprintf(fp, "<br/>\n");
+  iter = kwv_iter_begin(kwv, NULL);
+  while ((pair = kwv_iter_next(iter)) != NULL) {
+    attribute_hl(fp, NULL, pair->name, pair->val);
+	fprintf(fp, "<br/>");
+  }
+  kwv_iter_end(iter);
+}
+
 int
 main(int argc, char **argv)
 {
   int mime_arg, html, i, onevalue, shell, suppress_newline, arg_test;
-  int debug_flag;
-  char *infile, *p, *query, *query_string;
+  int check_dup, debug_flag, dup_error, saw_mode_arg;
+  char *infile, *p, *query, *query_string, *remote_addr;
   Cgiparse_enc enc_type;
   Kwv *kwv;
+  Kwv_dup_mode dup_mode;
   Log_desc *ld;
   Mime *mime;
 
+  saw_mode_arg = 0;
   suppress_newline = 0;
   arg_test = 0;
   html = 0;
   shell = 0;
   onevalue = 0;
+  check_dup = 0;
   query = NULL;
   query_string = getenv("QUERY_STRING");
   enc_type = CGIPARSE_ENC_NONE;
@@ -673,6 +707,14 @@ main(int argc, char **argv)
   mime = NULL;
   debug_flag = 0;
 
+  remote_addr = getenv("REMOTE_ADDR");
+
+  /*
+   * Allow duplicate parameter names.
+   */
+  dup_mode = KWV_ALLOW_DUPS;
+  dup_error = 0;
+
   /*
    * XXX add a mode where all varname/varvalue pairs are emitted;
    * varvalues need to be encoded
@@ -684,10 +726,14 @@ main(int argc, char **argv)
 		usage();
 	  query = argv[i];
 	  html = 0;
+	  saw_mode_arg++;
 	}
-	else if (streq(argv[i], "-d")) {
+	else if (streq(argv[i], "-checkdup")) {
+	  check_dup++;
+	  saw_mode_arg++;
+	}
+	else if (streq(argv[i], "-d"))
 	  debug_flag++;
-	}
 	else if (streq(argv[i], "-enc")) {
 	  if (++i == argc)
 		usage();
@@ -702,35 +748,53 @@ main(int argc, char **argv)
 	  else
 		usage();
 	}
-	else if (streq(argv[i], "-html"))
+	else if (streq(argv[i], "-html")) {
 	  html = 1;
+	  saw_mode_arg++;
+	}
 	else if (streq(argv[i], "-in")) {
 	  if (++i == argc)
 		usage();
 	  infile = argv[i];
 	}
+	else if (streq(argv[i], "-nodups"))
+	  dup_mode = KWV_REPLACE_DUPS;
+	else if (streq(argv[i], "-duperror")) {
+	  dup_error = 1;
+	  dup_mode = KWV_NO_DUPS;
+	}
 	else if (streq(argv[i], "-nonewline"))
 	  suppress_newline = 1;
 	else if (streq(argv[i], "-one")) {
 	  onevalue = 1;
 	  html = 0;
+	  saw_mode_arg++;
 	}
 	else if (streq(argv[i], "-qs")) {
 	  if (++i == argc)
 		usage();
 	  query_string = argv[i];
 	}
-	else if (streq(argv[i], "-sh"))
+	else if (streq(argv[i], "-sh") || streq(argv[i], "--shell")) {
 	  shell = 1;
+	  saw_mode_arg++;
+	}
 	else if (streq(argv[i], "-targ")) {
 	  if (++i == argc)
 		usage();
 	  arg_test = 1;
 	  query = argv[i];
 	  html = 0;
+	  saw_mode_arg++;
 	}
-	else if (streq(argv[i], "-text"))
+	else if (streq(argv[i], "-text")) {
 	  html = 0;
+	  saw_mode_arg++;
+	}
+	else if (streq(argv[i], "--version")) {
+	  dacs_version(stderr);
+	  exit(0);
+	}
 	else
 	  break;
   }
@@ -741,6 +805,13 @@ main(int argc, char **argv)
 	log_set_desc(ld, LOG_DISABLED);
   }
 
+  /* If there is no mode argument, -text is the default. */
+  if (saw_mode_arg > 1) {
+	fprintf(stderr, "Too many mode arguments\n");
+	usage();
+	/*NOTREACHED*/
+  }
+
   mime_arg = i;
   while (i < argc) {
 	if (streq(argv[i], "-copy")) {
@@ -760,9 +831,16 @@ main(int argc, char **argv)
 	i++;
   }
 
-  if (html)
+  if (html && remote_addr != NULL)
 	emit_html_header(stdout, NULL);
 
+  kwv = kwv_init(100);
+  kwv->dup_mode = dup_mode;
+
+  /*
+   * If an input file was specified, read name/value pairs from it,
+   * otherwise parse a query string read from stdin.
+   */
   if (infile != NULL) {
 	Ds *ds;
 	Kwv_conf conf = {
@@ -782,25 +860,36 @@ main(int argc, char **argv)
 	  exit(1);
 	}
 
-	if ((kwv = kwv_make_new(ds_buf(ds), &conf)) == NULL) {
+	if (kwv_make_sep(kwv, ds_buf(ds), &conf) == NULL) {
 	  if (html)
-		printf("<p>Fatal cgiparse error.\n");
+		printf("<p>Fatal parameter parsing error.\n");
 	  else
-		fprintf(stderr, "Fatal error\n");
+		fprintf(stderr, "Fatal parameter parsing error\n");
 	  exit(1);
 	}
   }
   else {
-	kwv = kwv_init(100);
 	if (cgiparse(stdin, query_string, kwv, &mime) == -1) {
 	  if (html)
-		printf("<p>Fatal cgiparse error.\n");
+		printf("<p>Fatal parameter parsing error.\n");
 	  else
-		fprintf(stderr, "Fatal error\n");
+		fprintf(stderr, "Fatal parsing error\n");
 	  exit(1);
 	}
   }
 
+  if (check_dup) {
+	for (i = 0; i < kwv->nused; i++) {
+	  if (kwv->pairs[i]->next != NULL)
+		exit(1);
+	}
+
+	exit(0);
+  }
+
+  /*
+   * In the query mode, only one specified parameter name is considered.
+   */
   if (query != NULL) {
 	Kwv_pair *pair;
 
@@ -845,74 +934,92 @@ main(int argc, char **argv)
 	exit(1);
   }
 
+  /*
+   * In the non-query modes, all parameters are output in some fashion.
+   */
   if (kwv->nused > 0) {
 	if (html) {
 	  printf("CGI parameters:<br>");
-	  kwv_html(stdout, kwv);
+	  emit_html(stdout, kwv);
 	}
 	else if (onevalue) {
-	  for (i = 0; i < kwv->nused; i++) {
-		if (*kwv->pairs[i]->val == '\0' && kwv->pairs[i]->xval != NULL)
-		  p = kwv->pairs[i]->xval;
-		else
-		  p = kwv->pairs[i]->val;
+	  Kwv_iter *iter;
+	  Kwv_pair *pair;
 
+	  iter = kwv_iter_begin(kwv, NULL);
+	  while ((pair = kwv_iter_next(iter)) != NULL) {
+		if (pair->val[0] == '\0' && pair->xval != NULL)
+		  p = pair->xval;
+		else
+		  p = pair->val;
 		printf("%s\n", p);
 	  }
+	  kwv_iter_end(iter);
 	}
 	else if (shell) {
-	  char *n;
+	  char *varname;
 	  Ds *ds;
+	  Kwv_iter *iter;
+	  Kwv_pair *pair;
 
-	  for (i = 0; i < kwv->nused; i++) {
-		n = kwv->pairs[i]->name;
-		if (*kwv->pairs[i]->val == '\0' && kwv->pairs[i]->xval != NULL)
-		  p = kwv->pairs[i]->xval;
+	  iter = kwv_iter_begin(kwv, NULL);
+	  while ((pair = kwv_iter_next(iter)) != NULL) {
+		varname = pair->name;
+		if (pair->val[0] == '\0' && pair->xval != NULL)
+		  p = pair->xval;
 		else
-		  p = kwv->pairs[i]->val;
+		  p = pair->val;
 
 		if ((ds = decode_val(p, enc_type)) == NULL)
 		  exit(1);
 	  
-		if ((p = shell_assign(n, ds)) == NULL)
+		if ((p = shell_assign(varname, ds)) == NULL)
 		  exit(1);
 
 		printf("%s", p);
 	  }
+	  kwv_iter_end(iter);
 	  printf("\n");
 	}
 	else {
-	  for (i = 0; i < kwv->nused; i++) {
-		if (*kwv->pairs[i]->val == '\0' && kwv->pairs[i]->xval != NULL) {
-		  p = kwv->pairs[i]->xval;
-		  printf("%s %s\n", kwv->pairs[i]->name, p);
+	  Kwv_iter *iter;
+	  Kwv_pair *pair;
+
+	  iter = kwv_iter_begin(kwv, NULL);
+	  while ((pair = kwv_iter_next(iter)) != NULL) {
+		if (pair->val[0] == '\0' && pair->xval != NULL) {
+		  p = pair->xval;
+		  printf("%s %s\n", pair->name, p);
 		  continue;
 		}
 
 		switch (enc_type) {
 		case CGIPARSE_ENC_URL:
-		  p = url_encode(kwv->pairs[i]->val, 0);
+		  p = url_encode(pair->val, 0);
 		  break;
+
 		case CGIPARSE_ENC_M64:
-		  mime_encode_base64((unsigned char *) kwv->pairs[i]->val,
-							 strlen(kwv->pairs[i]->val), &p);
+		  mime_encode_base64((unsigned char *) pair->val,
+							 strlen(pair->val), &p);
 		  break;
+
 		case CGIPARSE_ENC_D64:
-		  strba64((unsigned char *) kwv->pairs[i]->val,
-				  strlen(kwv->pairs[i]->val), &p);
+		  strba64((unsigned char *) pair->val, strlen(pair->val), &p);
 		  break;
+
 		case CGIPARSE_ENC_NONE:
 		default:
-		  p = kwv->pairs[i]->val;
+		  p = pair->val;
 		  break;
 		}
 
-		printf("%s %s\n", kwv->pairs[i]->name, p);
+		printf("%s %s\n", pair->name, p);
 	  }
+	  kwv_iter_end(iter);
 	}
   }
 
-  if (html)
+  if (html && remote_addr != NULL)
 	emit_html_trailer(stdout);
 
   if (debug_fp != NULL)
