@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2015
+ * Copyright (c) 2003-2017
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -43,10 +43,10 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2015\n\
+"Copyright (c) 2003-2017\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: conf.c 2827 2015-07-23 23:37:16Z brachman $";
+  "$Id: conf.c 2970 2017-07-06 18:00:25Z brachman $";
 #endif
 
 #include "dacs.h"
@@ -2032,6 +2032,8 @@ process_conf(char *conf_file, char *site_conf_file, Kwv_vartab *vartab,
 	kwv_add(site->kwv_dacs, "INTERACTIVE", "1");
 
   acs_new_env(&env);
+  env.is_dacsexpr = 0;
+
   conf_var_ns = var_ns_new(&env.namespaces, "Conf", site->kwv_conf);
   if (dacs_kwv_args != NULL)
 	var_ns_new(&env.namespaces, "Args", dacs_kwv_args);
@@ -2048,7 +2050,8 @@ process_conf(char *conf_file, char *site_conf_file, Kwv_vartab *vartab,
 	log_msg((LOG_ERROR_LEVEL, "Parse of config file '%s' failed", conf_file));
 	return(NULL);
   }
-  else {
+
+  {
 	Jurisdiction_conf *cp;
 	Acs_expr_result st;
 	Expr_result result;
@@ -2146,7 +2149,7 @@ process_conf(char *conf_file, char *site_conf_file, Kwv_vartab *vartab,
   kwv_conf.eval_func = conf_eval;
   kwv_conf.eval_arg = &eval_arg;
   if ((kwv = kwv_make_new(ds_buf(&j->string), &kwv_conf)) == NULL) {
-	log_msg((LOG_ERROR_LEVEL, "In Jurisdiction section %s, error processing %s",
+	log_msg((LOG_ERROR_LEVEL, "Jurisdiction section %s: error processing %s",
 			 j_id, arg));
 	return(NULL);
   }
@@ -2177,7 +2180,7 @@ process_conf(char *conf_file, char *site_conf_file, Kwv_vartab *vartab,
 	 */
 	if (conf_merge(kwv, default_kwv, conf_vartab, &eval_arg) == NULL) {
 	  log_msg((LOG_ERROR_LEVEL,
-			   "In Jurisdiction section %s, error merging defaults", j_id));
+			   "Jurisdiction section %s: error merging defaults", j_id));
 	  return(NULL);
 	}
   }
@@ -2189,7 +2192,7 @@ process_conf(char *conf_file, char *site_conf_file, Kwv_vartab *vartab,
 	 */
 	if (conf_merge(kwv, site->kwv_site, conf_vartab, &eval_arg) == NULL) {
 	  log_msg((LOG_ERROR_LEVEL,
-			   "In Jurisdiction section %s, error merging site defaults",
+			   "Jurisdiction section %s: Error merging site defaults",
 			   j_id));
 	  return(NULL);
 	}
@@ -2203,7 +2206,7 @@ process_conf(char *conf_file, char *site_conf_file, Kwv_vartab *vartab,
 	kwv_vartab_init(kwv, vartab, conf_site, (void *) site);
 	if (dacs_app_type != DACS_LOCAL_SERVICE
 		&& kwv_vartab_check(kwv, vartab, &kw, &err_vt, &errmsg) == -1) {
-	  log_msg((LOG_ERROR_LEVEL, "In Jurisdiction section %s:", j_id));
+	  log_msg((LOG_ERROR_LEVEL, "Jurisdiction section %s:", j_id));
 	  if (err_vt != NULL)
 		log_msg((LOG_ERROR_LEVEL, "Config error: %s: %s",
 				 errmsg, err_vt->name));
@@ -2217,7 +2220,7 @@ process_conf(char *conf_file, char *site_conf_file, Kwv_vartab *vartab,
 
   if ((kwv = conf_resolve(kwv, site, conf_vartab, &eval_arg)) == NULL) {
 	log_msg((LOG_ERROR_LEVEL,
-			 "In Jurisdiction section %s, failed to resolve merged config",
+			 "Jurisdiction section %s: Failed to resolve merged config",
 			 j_id));
 	return(NULL);
   }
@@ -2247,8 +2250,7 @@ process_conf(char *conf_file, char *site_conf_file, Kwv_vartab *vartab,
 	  if (process_auth_sections(conf->default_auth_conf, &env,
 								site, dc->auth_vartab) == -1) {
 		log_msg((LOG_ERROR_LEVEL,
-				 "In Default Jurisdiction section %s, invalid Auth section",
-				 j_id));
+				 "Default Jurisdiction section %s: Invalid <Auth>", j_id));
 		return(NULL);
 	  }
 	}
@@ -2257,7 +2259,7 @@ process_conf(char *conf_file, char *site_conf_file, Kwv_vartab *vartab,
 	  if (process_auth_sections(site->site_conf->default_auth_conf, &env,
 								site, dc->auth_vartab) == -1) {
 		log_msg((LOG_ERROR_LEVEL,
-				 "In Site Jurisdiction section %s, invalid Default section",
+				 "Site Jurisdiction section %s: Invalid <Default>",
 				 j_id));
 		return(NULL);
 	  }
@@ -2269,14 +2271,14 @@ process_conf(char *conf_file, char *site_conf_file, Kwv_vartab *vartab,
 	  if (process_auth_sections(conf->default_auth_conf, &env,
 								site, NULL) == -1) {
 		log_msg((LOG_ERROR_LEVEL,
-				 "In Default Jurisdiction section %s, invalid Auth config",
+				 "Default Jurisdiction section %s: Invalid <Auth> config",
 				 j_id));
 		return(NULL);
 	  }
 	}
 	if (process_auth_sections(auth_conf, &env, site, dc->auth_vartab) == -1) {
 	  log_msg((LOG_ERROR_LEVEL,
-			   "In Jurisdiction section %s, invalid Auth config", j_id));
+			   "Jurisdiction section %s: Invalid <Auth> config", j_id));
 	  return(NULL);
 	}
   }
@@ -2290,7 +2292,7 @@ process_conf(char *conf_file, char *site_conf_file, Kwv_vartab *vartab,
 	  if (process_roles_sections(conf->default_roles_conf, &env,
 								site, dc->roles_vartab) == -1) {
 		log_msg((LOG_ERROR_LEVEL,
-				 "In Jurisdiction section %s, invalid Default section", j_id));
+				 "Jurisdiction section %s: Invalid <Default>", j_id));
 		return(NULL);
 	  }
 	}
@@ -2299,7 +2301,7 @@ process_conf(char *conf_file, char *site_conf_file, Kwv_vartab *vartab,
 	  if (process_roles_sections(site->site_conf->default_roles_conf, &env,
 								 site, dc->roles_vartab) == -1) {
 		log_msg((LOG_ERROR_LEVEL,
-				 "In Jurisdiction section %s, invalid Site Default section",
+				 "Jurisdiction section %s: Invalid Site <Default>",
 				 j_id));
 		return(NULL);
 	  }
@@ -2311,7 +2313,7 @@ process_conf(char *conf_file, char *site_conf_file, Kwv_vartab *vartab,
 	  if (process_roles_sections(conf->default_roles_conf, &env,
 								site, NULL) == -1) {
 		log_msg((LOG_ERROR_LEVEL,
-				 "In Jurisdiction section %s, invalid Roles config in Default section",
+				 "Jurisdiction section %s: Invalid Roles config in <Default>",
 				 j_id));
 		return(NULL);
 	  }
@@ -2319,7 +2321,7 @@ process_conf(char *conf_file, char *site_conf_file, Kwv_vartab *vartab,
 	if (process_roles_sections(roles_conf, &env, site, dc->roles_vartab)
 		== -1) {
 	  log_msg((LOG_ERROR_LEVEL,
-			   "In Jurisdiction section %s, invalid Roles config in Jurisdiction section",
+			   "Jurisdiction section %s: Invalid Roles config in <Jurisdiction>",
 			   j_id));
 	  return(NULL);
 	}
@@ -2334,7 +2336,7 @@ process_conf(char *conf_file, char *site_conf_file, Kwv_vartab *vartab,
 	  if (process_transfer_sections(conf->default_transfer_conf, &env,
 									site, dc->transfer_vartab) == -1) {
 		log_msg((LOG_ERROR_LEVEL,
-				 "In Jurisdiction section %s, invalid Default section",
+				 "Jurisdiction section %s: Invalid <Default>",
 				 j_id));
 		return(NULL);
 	  }
@@ -2344,8 +2346,7 @@ process_conf(char *conf_file, char *site_conf_file, Kwv_vartab *vartab,
 	  if (process_transfer_sections(site->site_conf->default_transfer_conf,
 									&env, site, dc->transfer_vartab) == -1) {
 		log_msg((LOG_ERROR_LEVEL,
-				 "In Jurisdiction section %s, invalid Site Default section",
-				 j_id));
+				 "Jurisdiction section %s: Invalid Site <Default>", j_id));
 		return(NULL);
 	  }
 	}
@@ -2356,7 +2357,7 @@ process_conf(char *conf_file, char *site_conf_file, Kwv_vartab *vartab,
 	  if (process_transfer_sections(conf->default_transfer_conf, &env,
 								site, NULL) == -1) {
 		log_msg((LOG_ERROR_LEVEL,
-				 "In Jurisdiction section %s, invalid Transfer config in Default section",
+				 "Jurisdiction section %s: Invalid Transfer config in <Default>",
 				 j_id));
 		return(NULL);
 	  }
@@ -2364,7 +2365,7 @@ process_conf(char *conf_file, char *site_conf_file, Kwv_vartab *vartab,
 	if (process_transfer_sections(transfer_conf, &env, site,
 								  dc->transfer_vartab) == -1) {
 	  log_msg((LOG_ERROR_LEVEL,
-			   "In Jurisdiction section %s, invalid Transfer config in Jurisdiction section",
+			   "Jurisdiction section %s: Invalid Transfer config in <Jurisdiction>",
 			   j_id));
 	  return(NULL);
 	}
