@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2016
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -43,10 +43,10 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2012\n\
+"Copyright (c) 2003-2016\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: current_credentials.c 2594 2012-10-19 17:28:49Z brachman $";
+  "$Id: current_credentials.c 2931 2016-11-23 18:51:55Z brachman $";
 #endif
 
 #include "dacs.h"
@@ -255,7 +255,7 @@ main(int argc, char **argv)
 			}
 		  }
 		  else
-			printf("<br>There is no record of a previous sign on for this identity");
+			printf("<br>There is no record of a previous sign on for this identity.");
 
 		  if (dsvec_len(dsv_active) > 1) {
 			time_t expires_secs;
