@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2016
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -10,16 +10,16 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2012\n\
+"Copyright (c) 2003-2016\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: dacscheck.c 2642 2013-02-27 19:48:45Z brachman $";
+  "$Id: dacscheck.c 2902 2016-07-05 14:45:35Z brachman $";
 #endif
 
-#include <pwd.h>
-
 #include "dacs_api.h"
 
+#include <pwd.h>
+
 static MAYBE_UNUSED char *log_module_name = "dacscheck";
 
 #ifndef PROG
@@ -83,6 +83,7 @@ dacs_usage(void)
   fprintf(stderr, "-rules rules_vfs: add rules_vfs to DACS rulesets\n");
   fprintf(stderr, "-v:               increase verbosity level\n");
   fprintf(stderr, "-var name=value:  set name to value in DACS context\n");
+  fprintf(stderr, "--version:        print version and exit\n");
   fprintf(stderr, "-vfs vfs_uri:     add vfs_uri to the VFS configuration\n");
   fprintf(stderr, "--:               end of flags\n");
 
@@ -105,7 +106,7 @@ dacs_usage(void)
 int
 dacscheck_main(int argc, char **argv, int do_init, void *main_out)
 {
-  int dump_flag, i, st, add_unix_groups, quiet_flag, redirect_flag;
+  int dump_flag, i, st, add_unix_groups, redirect_flag;
   char *app, **constraints, *ident_flags, *r;
   char *errmsg, *field_sep, *groups_vfs, *objects, *roles_vfs, *rules_vfs;
   char *idents, *p, *unix_roles;
@@ -117,6 +118,8 @@ dacscheck_main(int argc, char **argv, int do_init, void *main_out)
 
   errmsg = "Internal error";
 
+  dacs_quiet_flag = 1;		/* The default. */
+
   if (dacs_init(DACS_STANDALONE_NOARGS, &argc, &argv, NULL, &errmsg) == -1) {
   fail:
 	fprintf(stderr, "%s\n", errmsg);
@@ -135,7 +138,6 @@ dacscheck_main(int argc, char **argv, int do_init, void *main_out)
   roles_vfs = NULL;
   app = NULL;
   add_unix_groups = 0;
-  quiet_flag = 0;
   verbose_level = 0;
   dump_flag = 0;
   ident_flags = NULL;
@@ -492,7 +494,7 @@ dacscheck_main(int argc, char **argv, int do_init, void *main_out)
 	  kwv_add(kwv_conf, "NAME_COMPARE", argv[i]);
 	}
 	else if (streq(argv[i], "-q"))
-	  quiet_flag = 1;
+	  dacs_quiet_flag = 1;
 	else if (streq(argv[i], "-roles")) {
 	  char *h;
 
@@ -521,7 +523,7 @@ dacscheck_main(int argc, char **argv, int do_init, void *main_out)
 		roles_vfs = argv[i];
 	}
 	else if (streq(argv[i], "-redirect"))
-	  redirect_flag = quiet_flag = 1;
+	  redirect_flag = dacs_quiet_flag = 1;
 	else if (streq(argv[i], "-rules")) {
 	  char *h, *r;
 	  static int num = 1;
@@ -576,6 +578,10 @@ dacscheck_main(int argc, char **argv, int do_init, void *main_out)
 		goto fail;
 	  }
 	}
+	else if (strcaseeq(argv[i], "--version")) {
+	  fprintf(stderr, "%s\n", dacs_version_string());
+	  exit(0);
+	}
 	else if (streq(argv[i], "-vfs")) {
 	  if (++i == argc) {
 		errmsg = "vfs_uri expected after -vfs";
@@ -674,14 +680,14 @@ dacscheck_main(int argc, char **argv, int do_init, void *main_out)
 		&& result.denial_reason == ACS_DENIAL_REASON_BY_SIMPLE_REDIRECT
 		&& result.redirect_action != NULL)
 	  printf("%s\n", result.redirect_action);
-	else if (!quiet_flag)
+	else if (!dacs_quiet_flag)
 	  printf("Access is denied\n");
 
 	return(-1);
   }
 
   if (st == 1) {
-	if (!quiet_flag) {
+	if (!dacs_quiet_flag) {
 	  if (constraints != NULL && constraints[0] != NULL) {
 		printf("Access is granted");
 		for (i = 0; constraints[i] != NULL; i++) {
@@ -700,7 +706,7 @@ dacscheck_main(int argc, char **argv, int do_init, void *main_out)
 	return(0);
   }
 
-  if (!quiet_flag)
+  if (!dacs_quiet_flag)
 	printf("Access testing failed\n");
 
   return(1);
