@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2016
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -51,10 +51,10 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2012\n\
+"Copyright (c) 2003-2016\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: dacsinfocard.c 2586 2012-03-15 16:21:40Z brachman $";
+  "$Id: dacsinfocard.c 2925 2016-11-09 17:01:21Z brachman $";
 #endif
 
 #include "icx.h"
@@ -86,23 +86,23 @@ static char *item_type = ITEM_TYPE_INFOCARDS;
  * dangling.
  */
 int
-ic_get_digest_algorithm(char **digest_name, Passwd_digest_alg *alg)
+ic_get_digest_algorithm(char **digest_name, Digest_desc *desc)
 {
   char *name;
-  Passwd_digest_alg a;
+  Digest_desc *dd;
 
   if (*digest_name != NULL)
 	name = *digest_name;
   else if ((name = conf_val(CONF_INFOCARD_DIGEST)) == NULL)
 	name = INFOCARD_DIGEST_DEFAULT_NAME;
 
-  if ((a = passwd_lookup_digest_algorithm(name)) == PASSWD_ALG_NONE) {
+  if ((dd = passwd_get_digest_algorithm(name)) == NULL) {
 	log_msg((LOG_ERROR_LEVEL,
 			 "An unrecognized digest method is configured: \"%s\"", name));
 	return(-1);
   }
 
-  *alg = a;
+  *desc = *dd;
   if (*digest_name == NULL)
 	*digest_name = strdup(name);
 
@@ -307,7 +307,7 @@ mic_format_xml_entry(Mic_entry *mic)
   case INFOCARD_AUTHTYPE_PASSWD:
 	authtype_str = "passwd";
 	sts_auth_el = ds_xprintf("<digest hash_alg=\"%s\">%s</digest>",
-							 passwd_lookup_digest_name(mic->sts_auth->alg),
+							 mic->sts_auth->desc->dt->name,
 							 (mic->sts_auth->digest == NULL)
 							 ? "" : mic->sts_auth->digest);
 	break;
@@ -437,7 +437,7 @@ mic_parse_entry(char *stored_entry)
 
   mic->sts_auth = sts_auth = ALLOC(Mic_sts_auth);
   sts_auth->authtype = INFOCARD_AUTHTYPE_NONE;
-  sts_auth->alg = PASSWD_ALG_NONE;
+  sts_auth->desc = NULL;
   sts_auth->digest = NULL;
   sts_auth->self_issued_ppid = NULL;
   sts_auth->thumbprint = NULL;
@@ -642,14 +642,14 @@ mic_register_entry(Vfs_handle *h, char *identity, char *card_id,
   new_entry_len = mime_encode_base64(ds_ucbuf(ds), ds_len(ds), &new_entry);
 
   log_msg((LOG_TRACE_LEVEL, "Storing identity key=\"%s\"", identity_key));
-  if (vfs_put(h, identity_key, new_entry, new_entry_len) == -1) {
+  if (vfs_put(h, identity_key, new_entry, new_entry_len + 1) == -1) {
 	log_msg((LOG_ERROR_LEVEL, "Cannot write entry for identity \"%s\"",
 			 identity));
 	return(-1);
   }
 
   log_msg((LOG_TRACE_LEVEL, "Storing card_id key=\"%s\"", card_id_key));
-  if (vfs_put(h, card_id_key, new_entry, new_entry_len) == -1) {
+  if (vfs_put(h, card_id_key, new_entry, new_entry_len + 1) == -1) {
 	log_msg((LOG_ERROR_LEVEL, "Cannot write entry for card_id \"%s\"",
 			 mic->card_id));
 	return(-1);
@@ -961,7 +961,7 @@ sic_init_entry(Sic_entry *oic, char *digest, char *username, Ic_token *token)
 	sic = oic;
 
   sic->state = IC_UNCHANGED;
-  sic->alg = PASSWD_ALG_NONE;
+  sic->desc = NULL;
   sic->username = username;
   sic->friendly_id = icx_friendly_identifier(token->ppid, 1);
   sic->pubkey_modulus = ds_buf(token->modulus);
@@ -1002,7 +1002,7 @@ sic_init_entry(Sic_entry *oic, char *digest, char *username, Ic_token *token)
 static Ds *
 sic_format_xml_entry(Sic_entry *sic)
 {
-  char *digest_name;
+  const char *digest_name;
   Ds *ds;
 
   ds = ds_init(NULL);
@@ -1010,7 +1010,7 @@ sic_format_xml_entry(Sic_entry *sic)
   ds_asprintf(ds, " state=\"%s\"",
 			  (sic->state == IC_DISABLED) ? "disabled" : "enabled");
   ds_asprintf(ds, " friendly_id=\"%s\"", ds_buf(sic->friendly_id));
-  if ((digest_name = passwd_lookup_digest_name(sic->alg)) == NULL)
+  if ((digest_name = sic->desc->dt->name) == NULL)
 	return(NULL);
   ds_asprintf(ds, " alg=\"%s\">", digest_name);
 
@@ -1099,7 +1099,7 @@ sic_parse_entry(char *stored_entry)
 	log_msg((LOG_ERROR_LEVEL, "Parse error: no \"alg\" attribute"));
 	return(NULL);
   }
-  if ((sic->alg = passwd_lookup_digest_algorithm(p)) == PASSWD_ALG_NONE) {
+  if ((sic->desc = passwd_get_digest_algorithm(p)) == NULL) {
     log_msg((LOG_ERROR_LEVEL, "Parse error: Unrecognized \"alg\" field: %s",
 			 p));
 	return(NULL);
@@ -1395,14 +1395,14 @@ ic_lookup_entry(Vfs_handle *h, char *xmlToken, Ic_config *conf)
   Icx_ctx *ctx;
   Ic_token *token;
   Mic_entry *mic;
-  Passwd_digest_alg alg;
+  Digest_desc dd;
   Sic_entry *sic;
 
   if (xmlToken == NULL)
 	return(NULL);
 
   digest_name = NULL;
-  if (ic_get_digest_algorithm(&digest_name, &alg) == -1) {
+  if (ic_get_digest_algorithm(&digest_name, &dd) == -1) {
 	log_msg((LOG_ERROR_LEVEL, "Digest configuration error"));
 	return(NULL);
   }
@@ -1695,7 +1695,7 @@ sic_delete_entry(Vfs_handle *h, char *hashstr, char *username)
  */
 static int
 sic_register_entry(Vfs_handle *h, Ic_token *token, char *hashstr,
-				  Passwd_digest_alg alg, char *sic_username,
+				  Digest_desc *desc, char *sic_username,
 				  Ic_state new_state, Ds *data)
 {
   int is_disabled;
@@ -1763,7 +1763,7 @@ sic_register_entry(Vfs_handle *h, Ic_token *token, char *hashstr,
 	is_disabled = 0;
   }
 
-  sic.alg = alg;
+  sic.desc = desc;
   sic.private = data;
 
   ds = sic_format_xml_entry(&sic);
@@ -1775,11 +1775,11 @@ sic_register_entry(Vfs_handle *h, Ic_token *token, char *hashstr,
    * two keys becomes invalid.
    */
   if (vfs_put(h, sic_make_key_from_hashstr(hashstr), enc_entry,
-			  enc_entry_len) == -1)
+			  enc_entry_len + 1) == -1)
 	return(-1);
 
   if (vfs_put(h, sic_make_key_from_username(sic_username), enc_entry,
-			  enc_entry_len) == -1)
+			  enc_entry_len + 1) == -1)
 	return(-1);
 
   log_msg(((Log_level) (LOG_INFO_LEVEL | LOG_AUDIT_FLAG),
@@ -1859,7 +1859,7 @@ sic_modify_entry(Vfs_handle *h, char *username, int ops, Ds *data)
   ds = sic_format_xml_entry(sic);
   enc_entry_len = mime_encode_base64(ds_ucbuf(ds), ds_len(ds), &enc_entry);
   st = vfs_put(h, sic_make_key_from_username(username), enc_entry,
-			   enc_entry_len);
+			   enc_entry_len + 1);
 
   return(st);
 }
@@ -3003,7 +3003,7 @@ dacsinfocard_main(int argc, char **argv, int do_init, void *main_out)
   Cookie *cookies;
   Credentials *credentials, *selected;
   Ic_token *token;
-  Passwd_digest_alg alg;
+  Digest_desc desc;
   DACS_app_type app_type;
   Ic_config *conf;
   Ic_op op;
@@ -3213,7 +3213,7 @@ dacsinfocard_main(int argc, char **argv, int do_init, void *main_out)
 	  goto done;
 	}
 
-	if (ic_get_digest_algorithm(&digest_name, &alg) == -1) {
+	if (ic_get_digest_algorithm(&digest_name, &desc) == -1) {
 	  errmsg = "Configuration error";
 		goto fail;
 	}
@@ -3288,7 +3288,7 @@ dacsinfocard_main(int argc, char **argv, int do_init, void *main_out)
   case IC_OP_REGISTER:
 	ic_username = ic_select_username(token, selected);
 
-	st = sic_register_entry(h, token, hashstr, alg, ic_username,
+	st = sic_register_entry(h, token, hashstr, &desc, ic_username,
 							IC_ENABLED, NULL);
 	if (st == 0)
 	  msg = "The self-issued InfoCard has been registered.";
