@@ -1,15 +1,20 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2015
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
 
+/*
+ * XXX the guts of some of these functions are being moved to misc.c because they
+ * are of general use.
+ */
+
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2012\n\
+"Copyright (c) 2003-2015\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: expr_file.c 2594 2012-10-19 17:28:49Z brachman $";
+  "$Id: expr_file.c 2837 2015-08-24 22:33:38Z brachman $";
 #endif
 
 #include "dacs_config.h"
@@ -33,6 +38,7 @@ static const char revid[] =
 
 #include "ds.h"
 #include "str.h"
+#include "misc.h"
 
 #ifdef DACS_OS_CYGWIN
 /* For utimes() */
@@ -121,13 +127,6 @@ do_basename(int argc, char **argv, char **result)
   return(1);
 }
 
-static int
-abs_mode(char *mode_str, mode_t *mode)
-{
-
-  return(strnum_b(mode_str, STRNUM_MODE_T, 8, mode, NULL));
-}
-
 /*
  * Limited to an absolute mode (octal number).
  */
@@ -137,19 +136,8 @@ do_chmod(int argc, char **argv, char **result)
   char *file, *mode_str;
   mode_t mode;
 
-  mode_str = argv[1];
-  file = argv[2];
-
-  if (abs_mode(mode_str, &mode) != -1) {
-	if (chmod(file, mode) == -1) {
-	  *result = strdup(strerror(errno));
-	  return(-1);
-	}
-  }
-  else {
-	*result = "Invalid mode";
+  if (chmod_file(argv[2], argv[1], result) == -1)
 	return(-1);
-  }
 
   *result = "1";
   return(1);
@@ -205,21 +193,10 @@ do_lstat(int argc, char **argv, char **result)
 static int
 do_mkdir(int argc, char **argv, char **result)
 {
-  char *dirname, *mode_str;
-  mode_t mode;
 
-  dirname = argv[1];
-  if ((mode_str = argv[2]) == NULL)
-	mode = 0755;
-  else if (abs_mode(mode_str, &mode) == -1) {
-	*result = "Invalid mode";
+  if (mkdir_file(argv[1], argv[2], result) == -1)
 	return(-1);
-  }
 
-  if (mkdir(dirname, mode) == -1) {
-	*result = strdup(strerror(errno));
-	return(-1);
-  }
   *result = "1";
 
   return(1);
@@ -536,40 +513,9 @@ stat_failed:
 static int
 do_touch(int argc, char **argv, char **result)
 {
-  int fd, st;
-  char *file, *mode_str;
-  mode_t mode;
-  struct stat statb;
 
-  file = argv[1];
-  mode_str = argv[2];
-
-  st = stat(file, &statb);
-  if (st == -1) {
-	if (errno != ENOENT) {
-	  *result = strdup(strerror(errno));
-	  return(-1);
-	}
-	/* Doesn't exist, create it. */
-	if (mode_str == NULL)
-	  mode = 0644;
-	else if (abs_mode(mode_str, &mode) == -1) {
-	  *result = "Invalid mode";
-	  return(-1);
-	}
-	fd = open(file, O_WRONLY | O_CREAT, mode);
-	if (fd == -1 || fstat(fd, &statb) != 0 || close(fd) != 0) {
-	  *result = strdup(strerror(errno));
-	  return(-1);
-	}
-  }
-  else {
-	/* Exists, update it. */
-	if (utimes(file, NULL) == -1) {
-	  *result = strdup(strerror(errno));
-	  return(-1);
-	}
-  }
+  if (touch_file(argv[1], argv[2], result) == -1)
+	return(-1);
 
   return(0);
 }
@@ -934,7 +880,7 @@ fs(char *filename, struct stat *statbuf)
 	hostname = (char *) malloc(MAXHOSTNAMELEN);
 	if (gethostname(hostname, MAXHOSTNAMELEN) == -1)
 	  return(NULL);
-	if ((p = index(hostname, '.')) != NULL)
+	if ((p = strchr(hostname, (int) '.')) != NULL)
 	  *p = '\0';
   }
 
@@ -943,11 +889,11 @@ fs(char *filename, struct stat *statbuf)
    * of the form "host:file", oth. it will look like "/dev/..."
    */
   if ((mntonname = getmntpt(filename, statbuf)) != NULL) { 
-	if ((p = index(mntonname, ':')) == NULL)
+	if ((p = strchr(mntonname, (int) ':')) == NULL)
 	  return(hostname);
 	else {
 	  *p = '\0';
-	  if ((p = index(mntonname, '.')) != NULL)
+	  if ((p = strchr(mntonname, (int) '.')) != NULL)
 		*p = '\0';
 	  return(mntonname);
 	}
