@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2015
+ * Copyright (c) 2003-2017
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -49,10 +49,10 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2015\n\
+"Copyright (c) 2003-2017\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: http.c 2861 2015-12-28 22:17:08Z brachman $";
+  "$Id: http.c 2968 2017-07-06 17:54:18Z brachman $";
 #endif
 
 #include "dacs_config.h"
@@ -1486,10 +1486,11 @@ send_request(Http *h)
 	}
   }
 
-  log_msg(((Log_level) (LOG_TRACE_LEVEL | LOG_SENSITIVE_FLAG),
-		   "Request:\n%s", ds_buf(&request)));
-  if (verbose_flag > 1)
-	fprintf(stderr, "Request:\n%s", ds_buf(&request));
+  if (!is_http_prog)
+	log_msg(((Log_level) (LOG_TRACE_LEVEL | LOG_SENSITIVE_FLAG),
+			 "Request headers:\n%s", ds_buf(&request)));
+  else if (verbose_flag > 1)
+	fprintf(stderr, "Request headers:\n%s", ds_buf(&request));
 
   rc = write_to_server(h, ds_buf(&request), ds_len(&request) - 1);
   if (rc != -1)
@@ -2601,20 +2602,22 @@ dacshttp_main(int argc, char **argv, int do_init, void *main_out)
   if (verbose_flag) {
 	Uri_query_arg *r;
 
-	fprintf(stderr, "Method: %s\n", http_method_to_string(h->method));
-	fprintf(stderr, "Scheme: %s\n", uri->scheme);
-	fprintf(stderr, "Host: %s\n", uri->host);
-	fprintf(stderr, "Port: %u\n", uri->port);
-	fprintf(stderr, "URL: %s\n", uri->uri);
+	fprintf(stderr, "Request summary:\n");
+	fprintf(stderr, "  Method: %s\n", http_method_to_string(h->method));
+	fprintf(stderr, "  Scheme: %s\n", uri->scheme);
+	fprintf(stderr, "  Host: %s\n", uri->host);
+	fprintf(stderr, "  Port: %u\n", uri->port);
+	fprintf(stderr, "  URL: %s\n", uri->uri);
 	if (h->content_type != NULL)
-	  fprintf(stderr, "Content-Type: %s\n", h->content_type);
+	  fprintf(stderr, "  Content-Type: %s\n", h->content_type);
 	if (uri->path != NULL)
-	  fprintf(stderr, "Path: %s\n", uri->path);
+	  fprintf(stderr, "  Path: %s\n", uri->path);
 	for (i = 0; i < dsvec_len(uri->query_args); i++) {
 	  r = dsvec_ptr_index(uri->query_args, i);
-	  fprintf(stderr, "Query Arg: %s=\"%s\"\n", r->name, r->value);
+	  fprintf(stderr, "  Query Arg: %s=\"%s\"\n", r->name, r->value);
 	}
-	fprintf(stderr, "User-Agent: %s\n", user_agent_string);
+	fprintf(stderr, "  User-Agent: %s\n", user_agent_string);
+	fprintf(stderr, "\n");
   }
 
   http_set_post_params(h, dsvec_len(post_params),
