@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2014
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -63,10 +63,10 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2012\n\
+"Copyright (c) 2003-2014\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: icx.c 2586 2012-03-15 16:21:40Z brachman $";
+  "$Id: icx.c 2696 2014-07-03 23:45:58Z brachman $";
 #endif
 
 #include "icx.h"
@@ -1113,7 +1113,11 @@ icx_canonicalize_xml(Ds *xml)
 	return(NULL);
   }
 
+#if LIBXML_VERSION <= 20800
   ds = ds_set(NULL, (char *) buf->buffer->content);
+#else
+  ds = ds_set(NULL, (char *) xmlBufContent(buf->buffer));
+#endif
 
   return(ds);
 }
