@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2016
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -65,10 +65,10 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2012\n\
+"Copyright (c) 2003-2016\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: kwv.c 2588 2012-03-15 20:01:54Z brachman $";
+  "$Id: kwv.c 2913 2016-10-18 19:55:39Z brachman $";
 #endif
 
 #include "dacs_config.h"
@@ -446,6 +446,24 @@ delete_pair(Kwv *kwv, Kwv_pair *head)
 
 }
 
+Kwv_pair *
+kwv_init_pair(Kwv_pair *opair)
+{
+  Kwv_pair *pair;
+
+  if (opair == NULL)
+	pair = ALLOC(Kwv_pair);
+  else
+	pair = opair;
+
+  pair->name = NULL;
+  pair->val = NULL;
+  pair->xval = NULL;
+  pair->next = NULL;
+
+  return(pair);
+}
+
 static Kwv_pair *
 make_pair(Kwv_pair *pair, char *key, char *val, void *xval, copy_t copy)
 {
@@ -1920,6 +1938,17 @@ kwv_vartab_check(Kwv *kwv, Kwv_vartab *vartab,
  * The iterator state is maintained so that an element returned by
  * kwv_iter_first() or kwv_iter_next() can be deleted (by kwv_delete()) without
  * breaking the state necessary for kwv_iter_next().
+ *
+ * XXX possible extension: add a bit flag to Kwv called "marked_flag" and a
+ * field to Kwv_pair called "mark".  By default, both fields are zero.
+ * Admin functions are provided to set and clear both fields (also, all Kwv_pair
+ * fields in a Kwv), to test a given pair, and to count marked/unmarked pairs.
+ * The pair iteration functions
+ * are changed so that if marked_flag is set in Kwv, traversal only selects the
+ * marked (or unmarked) pairs (possibly also matching a key).  This would allow
+ * pairs to be processed and marked, making it easier to notice a pair that had
+ * already been processed or that was not processed (e.g., such as the case of
+ * an error due to an unrecognized argument).
  */
 Kwv_iter *
 kwv_iter_begin(Kwv *kwv, char *key)
@@ -2373,6 +2402,8 @@ kwv_buf(Kwv *kwv, int sep_ch, int quote_ch)
 	if (pair->val != NULL)
 	  ds_asprintf(ds, "%s%s%s%s%s\n", pair->name, sep_str,
 				  quote_str, pair->val, quote_str);
+	else if (pair->xval != NULL)
+	  ds_asprintf(ds, "%s%s[xval]\n", pair->name, sep_str);
   }
 
   kwv_iter_end(iter);
@@ -2405,6 +2436,76 @@ kwv_keys(Kwv *kwv)
   return(dsv);
 }
 
+#ifdef NOTDEF
+/*
+ * XXX API TBD
+ * Not sure if this should be a generalized attribute name/value matching
+ * function or if it is even needed at all.
+ * It is sometimes useful to check, for example while parsing an XML/HTML
+ * document, if a given attribute name is present or if the attribute
+ * name and value match given values.
+ *
+ * Iterate through KWV and compare each pair to ATTR_STR.
+ * ATTR_STR is either an attribute name or an attribute-name '=' attribute-value
+ * string.  In the former case, the attribute value is not significant.
+ * The comparison is case-sensitive.
+ * Return 1 if there is match, 0 otherwise.
+ */
+int
+kwv_attr_eq(Kwv *kwv, const char *attr_str, int flags)
+{
+  int rc;
+  char *attr_name_endp, *attr_value_p;
+  size_t attr_name_len;
+  Kwv_iter *iter;
+  Kwv_pair *pair;
+
+  attr_name_endp = strchr(attr_str, (int) '=');
+  if (attr_name_endp != NULL) {
+	attr_value_p = attr_str + attr_name_endp + 1;
+	attr_name_len = attr_name_endp - attr_str;
+  }
+  else {
+	attr_value_p = NULL;
+	attr_name_len = strlen(attr_str);
+  }
+
+  iter = kwv_iter_begin(kwv, NULL);
+  rc = 0;
+  for (pair = kwv_iter_first(iter); pair != NULL; pair = kwv_iter_next(iter)) {
+	if (attr_value_p == NULL) {
+	  if (flags & KWV_ATTRNAME_ICASE)
+		rc = strcaseeq(pair->name, attr_str);
+	  else
+		rc = streq(pair->name, attr_str);
+	  if (rc == 1)
+		break;
+	}
+	else {
+	  if (flags & KWV_ATTRNAME_ICASE)
+		rc = strncaseeq(pair->name, attr_str, attr_name_len);
+	  else
+		rc = strneq(pair->name, attr_str, attr_name_len);
+	  if (rc == 0)
+		continue;
+
+	  if (flags & KWV_ATTRVALUE_ICASE)
+		rc = strncaseeq(pair->value, attr_value_p);
+	  else
+		rc = strneq(pair->name, attr_value_p);
+
+	  if (rc == 1)
+		break;
+	  }
+	}
+  }
+
+  kwv_iter_end(iter);
+
+  return(rc);
+}
+#endif
+
 /*
  * Convert KWV into a vector of pointers to kw=val strings, NULL terminated.
  * This is the format used by environ(7), for instance.
