@@ -1,11 +1,12 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2015
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
 
 /*
  * Generic HTTP authentication, such as for use against a Google (tm) account
+ * (deprecated by Google in 2012).
  *
  * See:
  *    http://code.google.com/apis/accounts/AuthForInstalledApps.html
@@ -13,10 +14,10 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2012\n\
+"Copyright (c) 2003-2015\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: local_http_auth.c 2594 2012-10-19 17:28:49Z brachman $";
+  "$Id: local_http_auth.c 2785 2015-02-25 20:52:17Z brachman $";
 #endif
 
 #include "dacs.h"
@@ -27,11 +28,15 @@ static const char *log_module_name = "local_http_auth";
  * Authenticate against a web-based authentication service.
  * Return 0 if authentication succeeds, -1 otherwise.
  *
+ * If the web service succeeds (status == 200) and returns a valid auth_reply
+ * document, and VALID_AUTH_REPLY is non-NULL, return the parsed document.
+ *
  * Possible enhancement: optional regex that must match reply for
  * auth to succeed (or fail) in addition to the 200 status code.
  */
 int
-local_http_auth(char *username, char *password, char *aux, Url_auth *url_auth)
+local_http_auth(char *username, char *password, char *aux, Url_auth *url_auth,
+				Auth_reply **valid_auth_reply)
 {
   int argnum, i, reply_len, status_code;
   char *cookies[2], *reply;
@@ -80,11 +85,54 @@ local_http_auth(char *username, char *password, char *aux, Url_auth *url_auth)
 				  argnum, (Http_params *) dsvec_base(dsv), NULL, cookies,
 				  &reply, &reply_len, &status_code,
 				  response_headers) == -1 || status_code != 200) {
-	if (reply != NULL)
+	if (reply != NULL) {
+	  log_msg((LOG_ERROR_LEVEL,
+			   "Web service failed with status %d and document:", status_code));
 	  log_msg((LOG_ERROR_LEVEL, "%s", reply));
+	}
 	return(-1);
   }
 
+  /*
+   * The web service request succeeded (the status was 200).
+   *
+   * If VALID_AUTH_REPLY is NULL, the caller does not care whether a document is
+   * present or if that document is valid, so authentication has succeeded.
+   *
+   * If VALID_AUTH_REPLY is non-NULL, an auth_reply document must be present,
+   * syntactically valid, and will indicate success or failure of
+   * authentication; a valid document will always be returned regardless of
+   * the final outcome.
+   * The caller must perform additional validation on the document, such as
+   * whether the returned username is suitable.
+   */
+  if (valid_auth_reply != NULL) {
+	Auth_reply *auth_reply;
+
+	*valid_auth_reply = NULL;
+	if (reply == NULL) {
+	  log_msg((LOG_ERROR_LEVEL, "No auth_reply document is present"));
+	  return(-1);
+	}
+
+	if (parse_xml_auth_reply(reply, &auth_reply) == -1) {
+	  log_msg((LOG_ERROR_LEVEL, "Invalid auth_reply document"));
+	  return(-1);
+	}
+
+	log_msg((LOG_DEBUG_LEVEL, "Valid auth_reply document is present"));
+	*valid_auth_reply = auth_reply;
+	if (auth_reply->ok == NULL) {
+	  log_msg((LOG_DEBUG_LEVEL,
+			   "auth_reply document says authentication failed"));
+	  return(-1);
+	}
+	log_msg((LOG_DEBUG_LEVEL,
+			 "auth_reply document says authentication succeeded"));
+  }
+  else
+	log_msg((LOG_DEBUG_LEVEL, "auth_reply document is not needed"));
+
   return(0);
 }
 
@@ -92,8 +140,9 @@ local_http_auth(char *username, char *password, char *aux, Url_auth *url_auth)
 int
 main(int argc, char **argv)
 {
-  int emitted_dtd, i;
-  char *errmsg, *jurisdiction, *username, *password, *aux;
+  int emitted_dtd, i, need_auth_reply, st;
+  char *aux, *errmsg, *jurisdiction, *username, *password, *use_auth_reply;
+  Auth_reply *auth_reply;
   Auth_reply_ok ok;
   Kwv *kwv;
   Url_auth url_auth;
@@ -101,6 +150,8 @@ main(int argc, char **argv)
   emitted_dtd = 0;
   errmsg = "internal";
   username = password = aux = jurisdiction = NULL;
+  use_auth_reply = NULL;
+  auth_reply = NULL;
 
   dacs_init_allow_dups_default = 0;
   if (dacs_init(DACS_LOCAL_SERVICE, &argc, &argv, &kwv, &errmsg) == -1) {
@@ -155,6 +206,8 @@ main(int argc, char **argv)
 		goto fail;
 	  }
 	}
+	else if (streq(kwv->pairs[i]->name, "USE_AUTH_REPLY"))
+	  use_auth_reply = kwv->pairs[i]->val;
 	else if (streq(kwv->pairs[i]->name, "USERNAME_PARAMETER")
 			 && url_auth.username_parameter == NULL)
 	  url_auth.username_parameter = kwv->pairs[i]->val;
@@ -182,7 +235,17 @@ main(int argc, char **argv)
 	goto fail;
   }
 
-  if (local_http_auth(username, password, aux, &url_auth) == -1) {
+  if (use_auth_reply != NULL
+	  && (strcaseeq(use_auth_reply, "yes") || strcaseeq(use_auth_reply, "on")))
+	need_auth_reply = 1;
+  else
+	need_auth_reply = 0;
+
+  if (need_auth_reply)
+	st = local_http_auth(username, password, aux, &url_auth, &auth_reply);
+  else
+	st = local_http_auth(username, password, aux, &url_auth, NULL);
+  if (st == -1) {
     errmsg = "Username/Password/Aux incorrect";
 	goto fail;
   }
@@ -192,12 +255,23 @@ main(int argc, char **argv)
   if (aux != NULL)
 	strzap(aux);
 
-  /* XXX Map it? */
-  ok.username = username;
+  if (auth_reply != NULL) {
+	ok.username = auth_reply->ok->username;
+	if (auth_reply->ok->lifetime != NULL && auth_reply->ok->lifetime[0] != '\0')
+	  ok.lifetime = auth_reply->ok->lifetime;
+	else
+	  ok.lifetime = kwv_lookup_value(kwv, "CREDENTIALS_LIFETIME_SECS");
+	ok.roles_reply = auth_reply->ok->roles_reply;
+  }
+  else {
+	/* XXX Map it? */
+	ok.username = username;
+
+	/* If this wasn't specified, dacs_authenticate will use the default. */
+	ok.lifetime = kwv_lookup_value(kwv, "CREDENTIALS_LIFETIME_SECS");
+	ok.roles_reply = NULL;
+  }
 
-  /* If this wasn't specified, dacs_authenticate will use the default. */
-  ok.lifetime = kwv_lookup_value(kwv, "CREDENTIALS_LIFETIME_SECS");
-  ok.roles_reply = NULL;
   printf("%s\n", make_xml_auth_reply_ok(&ok));
 
   emit_xml_trailer(stdout);
