@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2018
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -38,15 +38,18 @@
  ***************************************************************************/
 
 /* 
- * NTML authentication using Samba
+ * NTML authentication using libdsm or Samba
+ *
+ * https://msdn.microsoft.com/en-us/library/cc207842.aspx
+ * http://davenport.sourceforge.net/ntlm.html
  */
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2012\n\
+"Copyright (c) 2003-2018\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: local_ntlm_auth.c 2901 2016-07-05 14:42:04Z brachman $";
+  "$Id: local_ntlm_auth.c 2989 2018-01-26 00:38:32Z brachman $";
 #endif
 
 #include "dacs.h"
@@ -60,6 +63,11 @@ static const char revid[] =
 #include <search.h>
 #endif
 
+static const char *log_module_name = "local_ntlm_auth";
+
+#ifndef PROG
+
+#ifdef ENABLE_SAMBA
 /*
  * Samba includes
  * XXX yes, this is hideous; Samba just does not export a clean, useful
@@ -68,10 +76,8 @@ static const char revid[] =
  * especially true as Samba moved from 3.2 to 3.5.  So to get at the Samba
  * functions we need to assemble this brittle kludge-erama and hold our nose.
  * Perhaps there is a simpler way.
+ * [Years after that was written, LibDSM seemed to be the answer...]
  */
-#ifdef OLD
-#include "includes.h"
-#else
 #include "include/config.h"
 #include "../lib/replace/replace.h"
 #include "include/version.h"
@@ -104,11 +110,6 @@ extern NTSTATUS cli_full_connection(void **output_cli,
 									int flags, int signing_state, bool *retry);
 extern void cli_shutdown(void *cli);
 extern TALLOC_CTX *talloc_tos(void);
-#endif
-
-static const char *log_module_name = "local_ntlm_auth";
-
-#ifndef PROG
 
 /*
  * Microsoft Windows NTLM authentication using Samba
@@ -120,8 +121,9 @@ static const char *log_module_name = "local_ntlm_auth";
  * Return 0 if authentication succeeds, -1 otherwise.
  */
 int
-local_ntlm_auth(char *username, char *password, char *aux,
-				int smb_port, char *smb_server, char *smb_domain)
+local_ntlm_auth_samba(char *username, char *password, char *aux,
+					  int smb_port, char *smb_server, char *smb_domain,
+					  char *smb_workgroup)
 {
   int st;
   char *ntlm_password, *ntlm_username, *domain;
@@ -132,8 +134,8 @@ local_ntlm_auth(char *username, char *password, char *aux,
   char *myname;
   struct sockaddr_storage server_ip;
   bool in_client = lp_is_in_client();
-#elif SAMBA_VERSION_MAJOR == 3 \
-  && SAMBA_VERSION_MINOR == 0 \
+#elif SAMBA_VERSION_MAJOR == 3					\
+  && SAMBA_VERSION_MINOR == 0					\
   && SAMBA_VERSION_RELEASE > 22
   /* NOTE: This may not be compatible with versions prior to 3.0.25b. */
   struct in_addr server_ip;
@@ -236,6 +238,52 @@ local_ntlm_auth(char *username, char *password, char *aux,
 
   return(st);
 }
+#else
+#include "libdsm/libdsm.h"
+#endif
+
+/*
+ * Return 0 if authentication succeeded, -1 otherwise.
+ */
+int
+local_ntlm_auth(char *username, char *password, char *aux,
+				int smb_port, char *smb_server, char *smb_domain,
+				char *smb_workgroup)
+{
+  int rc;
+
+#if defined(ENABLE_SAMBA)
+  rc = local_ntlm_auth_samba(username, password, aux, smb_port, smb_server,
+							 smb_domain, smb_workgroup);
+#elif defined(ENABLE_LIBDSM)
+  {
+	/*
+	 * Since this function is in a library external to DACS, we ask it to
+	 * pass an error message so it can be logged if authentication fails.
+	 */
+	int is_guest;
+	char *errmsg;
+
+	errmsg = NULL;
+	is_guest = 0;
+	rc = local_ntlm_auth_libdsm(username, password, aux, smb_port, smb_server,
+								smb_domain, smb_workgroup, &is_guest, &errmsg);
+	if (rc == -1) {
+	  if (errmsg != NULL)
+		log_msg((LOG_ERROR_LEVEL, "%s", errmsg));
+	  else
+		log_msg((LOG_ERROR_LEVEL, "local_ntlm_auth_libdsm failed"));
+	}
+	else if (is_guest)
+	  log_msg((LOG_NOTICE_LEVEL, "Username \"%s\" is a guest account",
+			   username));
+  }
+#else
+#error "Either Samba or libdsm must be enabled"
+#endif
+
+  return(rc);
+}
 
 #else
 int
@@ -243,14 +291,17 @@ main(int argc, char **argv)
 {
   int emitted_dtd, i, samba_port;
   char *errmsg, *jurisdiction, *username, *password, *aux;
-  char *samba_domain, *samba_server;
+  char *samba_domain, *samba_server, *samba_workgroup;
   Auth_reply_ok ok;
   Kwv *kwv;
 
   errmsg = "internal";
   emitted_dtd = 0;
   username = password = aux = jurisdiction = NULL;
-  samba_server = samba_domain = NULL;
+  samba_server = NULL;
+
+  samba_domain = "";
+  samba_workgroup = "";
   samba_port = -1;
 
   if (dacs_init(DACS_LOCAL_SERVICE, &argc, &argv, &kwv, &errmsg) == -1) {
@@ -336,7 +387,7 @@ main(int argc, char **argv)
   }
 
   if (local_ntlm_auth(username, password, aux, samba_port,
-					  samba_server, samba_domain) == -1) {
+					  samba_server, samba_domain, samba_workgroup) == -1) {
 	errmsg = "Username/Password/Aux incorrect";
 	goto fail;
   }
