@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2015
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -45,10 +45,10 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2012\n\
+"Copyright (c) 2003-2015\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: local_passwd_auth.c 2594 2012-10-19 17:28:49Z brachman $";
+  "$Id: local_passwd_auth.c 2844 2015-08-24 22:50:16Z brachman $";
 #endif
 
 #include "dacs.h"
@@ -63,8 +63,8 @@ int
 local_passwd_auth(char *username, char *given_passwd, char *aux)
 {
   int st;
-  char *digest_name;
-  Passwd_digest_alg alg;
+  const char *digest_name;
+  Digest_desc *dd;
   Proc_lock *lock;
   Pw_entry *pw;
   Vfs_handle *h;
@@ -84,7 +84,7 @@ local_passwd_auth(char *username, char *given_passwd, char *aux)
   }
 
   digest_name = NULL;
-  if (passwd_get_digest_algorithm(&digest_name, &alg) == -1) {
+  if ((dd = passwd_get_digest_algorithm(digest_name)) == NULL) {
 	log_msg((LOG_ERROR_LEVEL, "Can't get digest algorithm"));
 	vfs_close(h);
 	st = -1;
@@ -102,10 +102,10 @@ local_passwd_auth(char *username, char *given_passwd, char *aux)
 	log_msg((LOG_ERROR_LEVEL, "vfs_close() failed"));
 
   st = -1;
-  if (pw->state == PW_DISABLED)
+  if (pw->state == PW_STATE_DISABLED)
 	log_msg((LOG_NOTICE_LEVEL, "Account is disabled for username \"%s\"",
 			 username));
-  else if (passwd_check_digest(alg, given_passwd, pw) != 1) {
+  else if (passwd_check_digest(dd, given_passwd, pw) != 1) {
 	log_msg((LOG_NOTICE_LEVEL,
 			 "Password comparison failed for username \"%s\"", username));
   }
