@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2015
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -13,10 +13,10 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2012\n\
+"Copyright (c) 2003-2015\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: local_simple_auth.c 2594 2012-10-19 17:28:49Z brachman $";
+  "$Id: local_simple_auth.c 2844 2015-08-24 22:50:16Z brachman $";
 #endif
 
 #include "dacs.h"
@@ -31,8 +31,8 @@ int
 local_simple_auth(char *username, char *given_passwd, char *aux)
 {
   int st;
-  char *digest_name;
-  Passwd_digest_alg alg;
+  const char *digest_name;
+  Digest_desc *dd;
   Pw_entry *pw;
   Vfs_handle *h;
 
@@ -51,7 +51,7 @@ local_simple_auth(char *username, char *given_passwd, char *aux)
   }
 
   digest_name = NULL;
-  if (passwd_get_digest_algorithm(&digest_name, &alg) == -1) {
+  if ((dd = passwd_get_digest_algorithm(digest_name)) == NULL) {
 	log_msg((LOG_ERROR_LEVEL, "Can't get digest algorithm"));
 	vfs_close(h);
 	return(-1);
@@ -67,10 +67,10 @@ local_simple_auth(char *username, char *given_passwd, char *aux)
 	log_msg((LOG_ERROR_LEVEL, "vfs_close() failed"));
 
   st = -1;
-  if (pw->state == PW_DISABLED)
+  if (pw->state == PW_STATE_DISABLED)
 	log_msg((LOG_NOTICE_LEVEL, "Account is disabled for username \"%s\"",
 			 username));
-  else if ((st = passwd_check_digest(alg, "", pw)) != 1)
+  else if ((st = passwd_check_digest(dd, "", pw)) != 1)
 	log_msg((LOG_NOTICE_LEVEL,
 			 "Password comparison failed for username \"%s\"", username));
   else {
