@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2014
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -34,22 +34,22 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2012\n\
+"Copyright (c) 2003-2014\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: mime.c 2610 2012-10-19 18:02:36Z brachman $";
+  "$Id: mime.c 2752 2014-12-31 17:54:01Z brachman $";
 #endif
 
 #ifdef DSSLIB
 #include "dsslib.h"
 #else
+#include "local.h"
+
 #include <unistd.h>
 #include <ctype.h>
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
-
-#include "local.h"
 #endif
 
 static const char *log_module_name = "mime";
@@ -944,10 +944,12 @@ parse_header(Mime *mime, Mime_part *part, char *field_name, char *token,
 int
 mime_parse_message_headers(Mime *mime, CGI_input *input)
 {
+  int header_linenum;
   char *field_name, *field_body, *line, *raw, *token;
   Mime_header *mh;
   Mime_parameter *param_list;
 
+  header_linenum = 1;
   while ((line = mime_next_header_line(input, &raw)) != NULL) {
 	/* A blank line ends the header area. */
 	if (*line == '\0')
@@ -957,8 +959,12 @@ mime_parse_message_headers(Mime *mime, CGI_input *input)
 	  mime->raw_mime_headers = dsvec_init(NULL, sizeof(char *));
 	dsvec_add_ptr(mime->raw_mime_headers, raw);
 
-	if (parse_header_field(line, &field_name, &field_body) == -1)
+	if (parse_header_field(line, &field_name, &field_body) == -1) {
+	  log_msg((LOG_ERROR_LEVEL,
+			   "parse_header_field failed at header line %d",
+				header_linenum));
 	  return(-1);
+	}
 
 	mh = ALLOC(Mime_header);
 	mh->field_name = field_name;
@@ -968,19 +974,28 @@ mime_parse_message_headers(Mime *mime, CGI_input *input)
 	dsvec_add_ptr(mime->mime_headers, mh);
 
 	if (strcaseeq(field_name, "MIME-Version")) {
-	  if (parse_header_field_body(field_body, &token, &param_list) == -1)
+	  if (parse_header_field_body(field_body, &token, &param_list) == -1) {
+		log_msg((LOG_ERROR_LEVEL,
+				 "parse_header_field_body failed at header line %d",
+				  header_linenum));
 		return(-1);
+	  }
 	  mime->version = token;
 	}
 	else if (strcaseeq(field_name, "Content-Type")) {
 	  if (parse_header_field_body(field_body, &token, &param_list) == -1) {
-		log_msg((LOG_ERROR_LEVEL, "Invalid Content-Type syntax: \"%s\"",
-				 strtrim(raw, "\n", 0)));
+		log_msg((LOG_ERROR_LEVEL,
+				 "Invalid Content-Type syntax (header line %d): \"%s\"",
+				 header_linenum, strtrim(raw, "\n", 0)));
 		return(-1);
 	  }
-	  if (do_content_type(&mime->content_type, token, param_list) == -1)
+	  if (do_content_type(&mime->content_type, token, param_list) == -1) {
+		log_msg((LOG_ERROR_LEVEL, "do_content_type failed at header line %d",
+				  header_linenum));
 		return(-1);
+	  }
 	}
+	header_linenum++;
   }
 
   if (line == NULL) {
@@ -1487,7 +1502,9 @@ mime_decode_base64(char *str, unsigned char **decoded)
 
 /*
  * RFC 2045 6.8
- * See also: RFC3548
+ * See also: RFC3548, RFC4648
+ *
+ * Return the length of the encoding, sans the null terminator.
  */
 long
 mime_encode_base64(unsigned char *str, unsigned int nbytes, char **encoded)
@@ -1510,12 +1527,14 @@ mime_encode_base64(unsigned char *str, unsigned int nbytes, char **encoded)
 	  rem = *p & 0x03;
 	  rem_bits = 2;
 	  break;
+
 	case 2:
 	  val = (rem << 4) | (*p & 0xf0) >> 4;
 	  ds_appendc(&ds, conv_table[val]);
 	  rem = *p & 0x0f;
 	  rem_bits = 4;
 	  break;
+
 	case 4:
 	  val = (rem << 2) | (*p & 0xc0) >> 6;
 	  ds_appendc(&ds, conv_table[val]);
@@ -1543,7 +1562,7 @@ mime_encode_base64(unsigned char *str, unsigned int nbytes, char **encoded)
   ds_appendc(&ds, '\0');
 
   *encoded = ds_buf(&ds);
-  return(ds_len(&ds));
+  return(ds_len(&ds) - 1);
 }
 
 int
