@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2016
+ * Copyright (c) 2003-2017
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -43,10 +43,10 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2016\n\
+"Copyright (c) 2003-2017\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: mkkey.c 2912 2016-10-18 19:54:07Z brachman $";
+  "$Id: mkkey.c 2971 2017-07-06 18:00:47Z brachman $";
 #endif
 
 #include "dacs.h"
@@ -58,14 +58,6 @@ static MAYBE_UNUSED const char *log_module_name = "dacskey";
 
 #ifndef PROG
 
-#ifdef NOTDEF
-/* The authentication key we are creating. */
-static unsigned char auth_key[AUTH_CRYPT_KEY_LENGTH];
-
-/* The HMAC key we are creating. */
-static unsigned char hmac_key[CRYPTO_HMAC_KEY_LENGTH];
-#endif
-
 static Crypt_keys *
 crypt_keys_init(void)
 {
@@ -135,15 +127,7 @@ mk_rsa(int rsa_key_bits, char **public_key, char **private_key)
 	  BN_set_bit(e, i);
   }
 
-#ifdef OLD
-  {
-	BN_GENCB cb;
-	BN_GENCB_set_old(&cb, NULL, NULL);
-	st = RSA_generate_key_ex(rsa, rsa_key_bits, e, &cb);
-  }
-#else
   st = RSA_generate_key_ex(rsa, rsa_key_bits, e, NULL);
-#endif
   BN_free(e);
 #else
   rsa = RSA_generate_key(rsa_key_bits, e_value, NULL, NULL);
