@@ -1,31 +1,71 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2016
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2012\n\
+"Copyright (c) 2003-2016\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: netlib.c 2586 2012-03-15 16:21:40Z brachman $";
+  "$Id: netlib.c 2877 2016-05-16 23:01:45Z brachman $";
 #endif
 
-#include <sys/ioctl.h>
-
 #ifdef DSSLIB
 #include "dsslib.h"
 #else
 #include "local.h"
 #endif
 
+#include <sys/ioctl.h>
+
 #if defined(DACS_OS_SOLARIS)
 #include <sys/filio.h>
 #endif
 
 static const char *log_module_name = "netlib";
 
+/* Originally, this was enabled, but it should change to disabled by default. */
+static int net_debug_flag = 1;
+
+/*
+ * Return the socket type (the second argument to socket(2)) of SD,
+ * or -1.
+ */
+int
+net_socket_type(int sd)
+{
+  int so_type;
+  socklen_t so_len;
+
+  so_len = sizeof(so_type);
+  if (getsockopt(sd, SOL_SOCKET, SO_TYPE, &so_type, &so_len) == -1) {
+	/* Assumed to be an invalid socket type. */
+	return(-1);
+  }
+
+  return(so_type);
+}
+
+int
+net_get_debug_flag(void)
+{
+
+  return(net_debug_flag);
+}
+
+int
+net_set_debug_flag(int flag)
+{
+  int prev_flag;
+
+  prev_flag = net_debug_flag;
+  net_debug_flag = flag;
+
+  return(prev_flag);
+}
+
 int
 net_tcp_nodelay(int sd)
 {
@@ -56,7 +96,8 @@ set_blocking_status(int sd, int block, char **errmsg)
   int flags;
 
   if (sd < 0) {
-	*errmsg = "Bad file descriptor";
+	if (errmsg != NULL)
+		*errmsg = "Bad file descriptor";
 	return(-1);
   }
 
@@ -87,6 +128,18 @@ set_blocking_status(int sd, int block, char **errmsg)
   return(0);
 }
 
+int
+net_set_min_input(int sd, int flag)
+{
+
+  if (setsockopt(sd, SOL_SOCKET, SO_RCVLOWAT, &flag, sizeof(flag)) == -1) {
+	log_err((LOG_ERROR_LEVEL, "setsockopt SO_RCVLOWAT"));
+	return(-1);
+  }
+
+  return(0);
+}
+
 /*
  * Set the given file descriptor to be non-blocking.
  * Return -1 on error, 0 otherwise.
@@ -147,6 +200,17 @@ net_write_str(int sd, char *mesg)
   return(net_write(sd, mesg, len));
 }
 
+int
+net_write_strln(int sd, char *mesg)
+{
+  size_t len;
+
+  len = strlen(mesg);
+  if (net_write(sd, mesg, len) == -1)
+	return(-1);
+  return(net_write(sd, "\n", 1));
+}
+
 /*
  * Assuming that STR is a domain name optionally followed by
  * a colon and a port number or service name, extract the port number (or
@@ -193,42 +257,117 @@ net_parse_hostname_port(char *str, char **hostname, char **portstr,
   return(rc);
 }
 
+/*
+ * Create a TCP/IP socket for HOSTNAME:PORT and return a (struct sockaddr_in *)
+ * for it, or NULL if an error occurs.
+ * XXX The first address returned by getaddrinfo() is used.
+ */
 struct sockaddr_in *
 net_make_sockaddr(char *hostname, in_port_t port)
 {
-  struct hostent *hp;
-  struct sockaddr_in *addr;
+  int gai_code;
+  char *port_str;
+  struct addrinfo ai_hints, *ai;
+  struct sockaddr_in *sai;
 
   if (hostname == NULL) {
 	log_msg((LOG_ERROR_LEVEL, "No hostname available?"));
 	return(NULL);
   }
 
-  addr = (struct sockaddr_in *) malloc(sizeof(struct sockaddr_in));
-  memset(addr, 0, sizeof(struct sockaddr_in));
-  addr->sin_family = AF_INET;
-  addr->sin_port = htons((u_short) port);
+  sai = (struct sockaddr_in *) malloc(sizeof(struct sockaddr_in));
+  memset(sai, 0, sizeof(struct sockaddr_in));
 
-  if ((hp = gethostbyname(hostname)) == NULL) {
-	in_addr_t numeric_addr;
+  port_str = ds_xprintf("%u", (unsigned int) port);
 
-	if ((numeric_addr = inet_addr(hostname)) == -1) {
-	  log_err((LOG_ERROR_LEVEL, "gethostbyname : Bad hostname: \"%s\"\n",
-			   hostname));
-	  return(NULL);
-	}
-	memcpy((void *) &addr->sin_addr, &numeric_addr, sizeof(numeric_addr));
+  memset(&ai_hints, 0, sizeof(ai_hints));
+  ai_hints.ai_family = AF_INET;
+  ai_hints.ai_socktype = SOCK_STREAM;
+  ai_hints.ai_protocol = IPPROTO_TCP;
+  ai_hints.ai_flags = AI_CANONNAME;
+
+  ai = NULL;
+  gai_code = getaddrinfo(hostname, port_str, &ai_hints, &ai);
+  if (gai_code != 0) {
+	const char *errmsg;
+
+	log_err((LOG_ERROR_LEVEL, "getaddrinfo: Bad hostname: \"%s\"\n",
+			 hostname));
+	errmsg = gai_strerror(gai_code);
+	log_msg((LOG_ERROR_LEVEL, "getaddrinfo: %s (%d)", errmsg, gai_code));
+	return(NULL);
   }
-  else
-	memcpy((void *) &addr->sin_addr, hp->h_addr, hp->h_length);
+  *sai = *(struct sockaddr_in *) ai->ai_addr;
 
-  endhostent();
+  freeaddrinfo(ai);
 
-  return(addr);
+  return(sai);
 }
 
 int
-net_connect_to_server_ssl(char *hostname, in_port_t port,
+net_tcp_connect(struct sockaddr_in *from, struct sockaddr_in *to,
+				int *read_fd, int *write_fd)
+{
+  int flag, sd;
+
+  if ((sd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
+	log_err((LOG_ERROR_LEVEL, "socket"));
+	return(-1);
+  }
+
+  flag = 1;
+  if (setsockopt(sd, SOL_SOCKET, TCP_NODELAY, &flag, sizeof(flag)) == -1) {
+	log_err((LOG_ERROR_LEVEL, "setsockopt TCP_NODELAY"));
+	close(sd);
+	return(-1);
+  }
+
+  flag = 1;
+  if (setsockopt(sd, SOL_SOCKET, SO_REUSEADDR, &flag, sizeof(flag)) == -1) {
+	log_err((LOG_ERROR_LEVEL, "setsockopt SO_REUSEADDR"));
+	close(sd);
+	return(-1);
+  }
+
+  if (from != NULL) {
+	if (bind(sd, (struct sockaddr *) from, sizeof(struct sockaddr_in)) == -1) {
+	  close(sd);
+	  log_err((LOG_ERROR_LEVEL, "bind"));
+	  return(-1);
+	}
+  }
+
+  log_msg((LOG_DEBUG_LEVEL, "Connecting to %s",
+		   net_sockaddr_name((struct sockaddr *) to,
+							 sizeof(struct sockaddr_in), NI_NUMERICSERV)));
+  if (connect(sd, (struct sockaddr *) to, sizeof(struct sockaddr_in)) == -1) {
+	log_err((LOG_ERROR_LEVEL, "connect"));
+	close(sd);
+	return(-1);
+  }
+
+  if (read_fd != NULL) {
+    *read_fd = sd;
+    if (write_fd != NULL) {
+      if ((*write_fd = dup(sd)) == -1) {
+        log_err((LOG_ERROR_LEVEL, "dup"));
+        close(sd);
+        return(-1);
+      }
+    }
+  }
+  else if (write_fd != NULL)
+    *write_fd = sd;
+  else {
+	close(sd);
+    return(-1);
+  }
+
+  return(0);
+}
+
+int
+net_connect_to_server_ssl(const char *hostname, in_port_t port,
 						  Net_connection_type ct,
 						  char *ssl_prog, char *ssl_prog_args,
 						  char *ssl_prog_client_crt,
@@ -296,18 +435,19 @@ net_connect_to_server_ssl(char *hostname, in_port_t port,
  * -1 otherwise.
  */
 int
-net_connect_to_server(char *hostname, in_port_t port,
+net_connect_to_server(const char *hostname, in_port_t port,
 					  int *read_fd, int *write_fd)
 {
-  int flag, sd;
-  struct sockaddr_in addr;
-  struct hostent *hp;
-  in_addr_t numeric_addr;
+  int flag, gai_code, sd;
+  char *port_str;
+  struct addrinfo ai_hints, *ai;
+  struct sockaddr_in *sai;
 
   if (hostname == NULL) {
 	log_msg((LOG_ERROR_LEVEL, "No hostname given"));
 	return(-1);
   }
+
   if (port == 0) {
 	log_msg((LOG_ERROR_LEVEL, "No port given"));
 	return(-1);
@@ -320,38 +460,52 @@ net_connect_to_server(char *hostname, in_port_t port,
 
   flag = 1;
   if (setsockopt(sd, SOL_SOCKET, TCP_NODELAY, &flag, sizeof(flag)) == -1) {
-	log_err((LOG_ERROR_LEVEL, "setsockopt"));
+	close(sd);
+	log_err((LOG_ERROR_LEVEL, "setsockopt TCP_NODELAY"));
 	return(-1);
   }
 
   flag = 1;
   if (setsockopt(sd, SOL_SOCKET, SO_REUSEADDR, &flag, sizeof(flag)) == -1) {
-	log_err((LOG_ERROR_LEVEL, "setsockopt2"));
+	log_err((LOG_ERROR_LEVEL, "setsockopt SO_REUSEADDR"));
 	close(sd);
 	return(-1);
   }
 
-  memset(&addr, 0, sizeof(addr));
-  addr.sin_family = AF_INET;
-  addr.sin_port = htons((u_short) port);
-  if ((hp = gethostbyname(hostname)) == NULL) {
-	if ((numeric_addr = inet_addr(hostname)) == -1) {
-	  log_msg((LOG_ERROR_LEVEL, "Unknown hostname: \"%s\"", hostname));
-	  close(sd);
-	  return(-1);
-	}
-	memcpy((char *) &addr.sin_addr, &numeric_addr, sizeof(numeric_addr));
+  port_str = ds_xprintf("%u", (unsigned int) port);
+
+  memset(&ai_hints, 0, sizeof(ai_hints));
+  ai_hints.ai_family = AF_INET;
+  ai_hints.ai_socktype = SOCK_STREAM;
+  ai_hints.ai_protocol = IPPROTO_TCP;
+  ai_hints.ai_flags = AI_CANONNAME;
+
+  ai = NULL;
+  gai_code = getaddrinfo(hostname, port_str, &ai_hints, &ai);
+  if (gai_code != 0) {
+	const char *errmsg;
+
+    errmsg = gai_strerror(gai_code);
+	log_msg((LOG_ERROR_LEVEL, "getaddrinfo: %s (%d): \"%s\"",
+			 errmsg, gai_code, hostname));
+	close(sd);
+	return(-1);
   }
-  else
-	memcpy((char *) &addr.sin_addr, hp->h_addr, hp->h_length);
 
-  log_msg((LOG_DEBUG_LEVEL, "Connecting to %s:%u", hostname, port));
+  sai = (struct sockaddr_in *) malloc(sizeof(struct sockaddr_in));
+  memset(sai, 0, sizeof(struct sockaddr_in));
+  *sai = *(struct sockaddr_in *) ai->ai_addr;
+  freeaddrinfo(ai);
 
-  if (connect(sd, (struct sockaddr *) &addr, sizeof(addr)) == -1) {
+  log_msg((LOG_DEBUG_LEVEL, "Connecting to %s:%s", hostname, port_str));
+
+  if (connect(sd, (struct sockaddr *) sai, sizeof(struct sockaddr_in)) == -1) {
 	log_err((LOG_ERROR_LEVEL, "connect"));
+	free(sai);
 	close(sd);
 	return(-1);
   }
+  free(sai);
 
   if (read_fd != NULL) {
 	*read_fd = sd;
@@ -365,8 +519,10 @@ net_connect_to_server(char *hostname, in_port_t port,
   }
   else if (write_fd != NULL)
 	*write_fd = sd;
-  else
+  else {
+	close(sd);
 	return(-1);
+  }
 
   return(0);
 }
@@ -388,7 +544,6 @@ net_input_or_timeout(int sd, struct timeval *timeout)
   FD_SET(sd, &fdset);
 
  again:
-
   if ((n = select(sd + 1, &fdset, NULL, NULL, timeout)) == -1) {
 	log_err((LOG_ERROR_LEVEL, "select"));
 	if (errno == EINTR) {
@@ -400,23 +555,39 @@ net_input_or_timeout(int sd, struct timeval *timeout)
   }
 
   if (n == 0) {
-	log_msg((LOG_ERROR_LEVEL, "Timeout"));
+	log_msg((LOG_TRACE_LEVEL, "Timeout"));
 	return(0);
   }
 
-  return(1);
+  /* Just in case... */
+  if (FD_ISSET(sd, &fdset))
+	return(1);
+
+  return(-1);
 }
 
 char *
-net_sockaddr_name(struct sockaddr *sa, socklen_t salen)
+net_addr(struct sockaddr_in *addr, in_port_t *portp)
+{
+  char *dot_addr;
+
+  dot_addr = strdup(inet_ntoa(addr->sin_addr));
+
+  if (portp != NULL)
+	*portp = (in_port_t) ntohs(addr->sin_port);
+
+  return(dot_addr);
+}
+
+char *
+net_sockaddr_name(struct sockaddr *sa, socklen_t salen, int flags)
 {
   char *p;
   char localhost[128], localport[16];
 
   if (getnameinfo(sa, salen,
 				  localhost, sizeof(localhost),
-				  localport, sizeof(localport),
-				  NI_NUMERICHOST | NI_NUMERICSERV) == 0)
+				  localport, sizeof(localport), flags) == 0)
 	p = ds_xprintf("%s:%s", localhost, localport);
   else
 	p = NULL;
@@ -476,7 +647,8 @@ net_socket_lname(int sd)
   }
 
   sname = net_sockaddr_name((struct sockaddr *) &addr,
-							sizeof(struct sockaddr_in));
+							sizeof(struct sockaddr_in),
+							NI_NUMERICHOST | NI_NUMERICSERV);
 
   return(sname);
 }
@@ -495,7 +667,8 @@ net_socket_rname(int sd)
   }
 
   sname = net_sockaddr_name((struct sockaddr *) &addr,
-							sizeof(struct sockaddr_in));
+							sizeof(struct sockaddr_in),
+							NI_NUMERICHOST | NI_NUMERICSERV);
 
   return(sname);
 }
@@ -542,18 +715,27 @@ net_get_service_port(char *portname, char *default_portname, char **errmsg)
   return(port);
 }
 
+/*
+ * Accept a connection on socket SD, but block for at most TIMEOUT
+ * (if TIMEOUT is NULL, wait indefinitely).
+ * Return the new socket descriptor and set FROM to the identity of the
+ * client if successful, -1 if an error occurs (including a timeout).
+ */
 int
 net_accept_or_timeout(int sd, struct sockaddr *from, struct timeval *timeout)
 {
-  int new_sd;
-  char *sname;
+  int new_sd, rc;
   socklen_t fromlen;
 
-  sname = net_socket_lname(sd);
-  log_msg((LOG_TRACE_LEVEL, "Waiting to accept connection to %s",
-		   (sname == NULL) ? "???" : sname));
+  if (log_would_log(LOG_TRACE_LEVEL)) {
+	char *sname;
 
-  if (net_input_or_timeout(sd, timeout) != 1)
+	sname = net_socket_lname(sd);
+	log_msg((LOG_TRACE_LEVEL, "Waiting to accept connection to %s",
+			 (sname == NULL) ? "???" : sname));
+  }
+
+  if ((rc = net_input_or_timeout(sd, timeout)) != 1)
 	return(-1);
 
   fromlen = sizeof(struct sockaddr);
@@ -562,16 +744,28 @@ net_accept_or_timeout(int sd, struct sockaddr *from, struct timeval *timeout)
 	return(-1);
   }
 
-  if ((sname = net_sockaddr_name(from, fromlen)) != NULL)
-	log_msg((LOG_TRACE_LEVEL, "Got new connection from %s", sname));
-  else
-	log_msg((LOG_TRACE_LEVEL, "Got new connection from ???"));
+  if (log_would_log(LOG_TRACE_LEVEL)) {
+	char *sname;
+
+	if ((sname = net_sockaddr_name(from, fromlen,
+								   NI_NUMERICHOST | NI_NUMERICSERV)) != NULL)
+	  log_msg((LOG_TRACE_LEVEL, "Got new connection from %s", sname));
+	else
+	  log_msg((LOG_TRACE_LEVEL, "Got new connection from ???"));
+  }
 
   return(new_sd);
 }
 
 /*
- * Create and configure a socket
+ * Create and configure a server-side socket for HOSTNAME:PORT.
+ * The address bound to the socket depends on the implementation of
+ * net_make_sockaddr().
+ * If successful, set SDP to the socket descriptor and NAMEP to the socket's
+ * (struct sockaddr_in), if NAMEP is non-NULL, and return 0; otherwise,
+ * return -1.
+ *
+ * XXX The backlog argument to listen(2) is zero.
  */
 int
 net_make_server_socket(char *hostname, in_port_t port,
@@ -589,26 +783,32 @@ net_make_server_socket(char *hostname, in_port_t port,
 
   flag = 1;
   if (setsockopt(sd, SOL_SOCKET, TCP_NODELAY, &flag, sizeof(flag)) == -1) {
-	log_err((LOG_ERROR_LEVEL, "setsockopt"));
+	close(sd);
+	log_err((LOG_ERROR_LEVEL, "setsockopt TCP_NODELAY"));
 	return(-1);
   }
 
   flag = 1;
   if (setsockopt(sd, SOL_SOCKET, SO_REUSEADDR, &flag, sizeof(flag)) == -1) {
-	log_err((LOG_ERROR_LEVEL, "setsockopt2"));
+	close(sd);
+	log_err((LOG_ERROR_LEVEL, "setsockopt SO_REUSEADDR"));
 	return(-1);
   }
 
-  if ((my_addr = net_make_sockaddr(hostname, port)) == NULL)
+  if ((my_addr = net_make_sockaddr(hostname, port)) == NULL) {
+	close(sd);
 	return(-1);
+  }
 
   if (bind(sd, (const struct sockaddr *) my_addr, sizeof(struct sockaddr_in))
 	  == -1) {
+	close(sd);
 	log_err((LOG_ERROR_LEVEL, "bind"));
 	return(-1);
   }
 
   if (listen(sd, 0) == -1) {
+	close(sd);
 	log_err((LOG_ERROR_LEVEL, "listen"));
 	return(-1);
   }
@@ -616,12 +816,14 @@ net_make_server_socket(char *hostname, in_port_t port,
   /* This is necessary to find out which local port has been assigned. */
   namelen = sizeof(struct sockaddr_in);
   if (getsockname(sd, (struct sockaddr *) my_addr, &namelen) == -1) {
+	close(sd);
     log_err((LOG_ERROR_LEVEL, "getsockname"));
     return(-1);
   }
 
   if ((sname = net_sockaddr_name((struct sockaddr *) my_addr,
-							   sizeof(struct sockaddr_in))) != NULL)
+								 sizeof(struct sockaddr_in),
+								 NI_NUMERICHOST | NI_NUMERICSERV)) != NULL)
 	log_msg((LOG_TRACE_LEVEL, "Created server socket at %s", sname));
 
   *sdp = sd;
@@ -629,3 +831,123 @@ net_make_server_socket(char *hostname, in_port_t port,
 
   return(0);
 }
+
+/*
+ * Receive a UDP message sent to socket SD (already bound to an address
+ * and port).
+ * If MAX_SECS is negative, poll (do not block), if it is zero block
+ * indefinitely, and if it is positive wait for up to that many seconds
+ * before timing out.
+ * The message is put in BUFP, the size in bytes of which is BUFLEN;
+ * BUFLEN is set to the actual length of the received message.
+ * The identity of the sender is returned in FROM and its size is FROMLEN.
+ *
+ * Return 0 if no message is available to be read, -1 if an error occurs,
+ * or 1 if a message is returned.
+ */
+int
+net_udp_recv(int sd, int max_secs, unsigned char *bufp, size_t *buflen,
+			 struct sockaddr *from, socklen_t *fromlen)
+{
+  int max_fds, st;
+  struct timeval timeout, *tv;
+  fd_set readfds;
+
+  if (log_would_log(LOG_DEBUG_LEVEL)) {
+	char *xxx;
+
+	xxx = net_socket_lname(sd);
+	log_msg((LOG_DEBUG_LEVEL, "Waiting for datagram on %s", xxx));
+  }
+
+  FD_ZERO(&readfds);
+  FD_SET(sd, &readfds);
+
+  if (max_secs < 0)
+	tv = NULL;
+  else {
+	timeout.tv_usec = 0;
+	timeout.tv_sec = (time_t) max_secs;
+	tv = &timeout;
+  }
+
+  max_fds = sd;
+  if ((st = select(max_fds + 1, &readfds, NULL, NULL, tv)) == -1) {
+	if (st == -1) {
+	  log_err((LOG_ERROR_LEVEL, "select"));
+	  return(-1);
+	}
+	if (st == 0)
+	  return(0);
+  }
+
+  if (FD_ISSET(sd, &readfds)) {
+	ssize_t len;
+
+	*fromlen = sizeof(struct sockaddr);
+	len = recvfrom(sd, bufp, *buflen, 0, from, fromlen);
+	if (len == -1) {
+	  log_err((LOG_ERROR_LEVEL, "recvfrom"));
+	  return(-1);
+	}
+	*buflen = (size_t) len;
+	log_msg((LOG_TRACE_LEVEL, "Read %u bytes [UDP]", (unsigned int) len));
+  }
+  else
+	return(0);
+
+  return(1);
+}
+
+/*
+ * Send the DATALEN byte UDP message in DATAP using socket SD.
+ * If TO is NULL, SD has been connected to its destination address, otherwise
+ * use TO.
+ * Return 0 if ok, otherwise -1.
+ */
+int
+net_udp_send(int sd, struct sockaddr_in *to, unsigned char *datap,
+			 size_t datalen)
+{
+  char *str;
+  in_port_t port;
+  ssize_t len;
+
+  if (to == NULL) {
+	if (log_would_log(LOG_TRACE_LEVEL)) {
+	  char *xxx, *yyy;
+
+	  xxx = net_socket_lname(sd);
+	  yyy = net_socket_rname(sd);
+	  log_msg((LOG_TRACE_LEVEL, "Sending datagram from %s to %s", xxx, yyy));
+	}
+
+	len = send(sd, datap, datalen, 0);
+  }
+  else {
+	if (log_would_log(LOG_TRACE_LEVEL)) {
+	  char *yyy;
+
+	  yyy = net_sockaddr_name((struct sockaddr *) to, sizeof(struct sockaddr_in),
+							  NI_NAMEREQD | NI_NUMERICSERV);
+	  log_msg((LOG_TRACE_LEVEL, "Sending datagram to %s", yyy));
+	}
+
+	len = sendto(sd, datap, datalen, 0, (struct sockaddr *) to,
+				 sizeof(struct sockaddr_in));
+  }
+
+  if (len != datalen) {
+	if (len == -1) {
+	  log_err((LOG_ERROR_LEVEL, "sendto"));
+	  return(-1);
+	}
+
+	log_msg((LOG_ERROR_LEVEL, "sendto: incomplete write"));
+	return(-1);
+  }
+
+  log_msg((LOG_TRACE_LEVEL, "Wrote %u bytes [UDP]", (unsigned int) len));
+
+  return(0);
+}
