@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2015
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -95,12 +95,15 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2012\n\
+"Copyright (c) 2003-2015\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: pamd-client.c 2594 2012-10-19 17:28:49Z brachman $";
+  "$Id: pamd-client.c 2765 2015-01-05 16:50:03Z brachman $";
 #endif
 
+#include "dacs.h"
+
+#ifdef HAVE_LIBPAM
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
@@ -109,8 +112,6 @@ static const char revid[] =
 #include <netdb.h>
 #include <security/pam_appl.h>
 
-#include "dacs.h"
-
 static const char *log_module_name = "pamd-client";
 
 static void
@@ -259,3 +260,12 @@ main(int argc, char **argv)
 
   exit(0);
 }
+#else
+int
+main(int argc, char **argv)
+{
+
+  fprintf(stderr, "PAM support is unavailable.\n");
+  exit(1);
+}
+#endif
