@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2015
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -206,12 +206,15 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2012\n\
+"Copyright (c) 2003-2015\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: pamd.c 2594 2012-10-19 17:28:49Z brachman $";
+  "$Id: pamd.c 2765 2015-01-05 16:50:03Z brachman $";
 #endif
 
+#include "dacs.h"
+
+#ifdef HAVE_LIBPAM
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
@@ -221,8 +224,6 @@ static const char revid[] =
 #include <security/pam_appl.h>
 #include <signal.h>
 
-#include "dacs.h"
-
 static const char *log_module_name = "pamd";
 
 static int http_flag;				/* Non-zero means run as a web service. */
@@ -237,19 +238,6 @@ static Pam_auth_tid *current_tid = NULL;
 static FILE *fp_in, *fp_err;
 static int fd_out = -1;
 
-/*
- * Older versions of pam_strerror() didn't take the pam_handle_t argument.
- */
-static const char *
-pam_strerr(pam_handle_t *h, int err)
-{
-#ifdef PAM_STRERROR_USES_PAMH
-  return(pam_strerror(h, err));
-#else
-  return(pam_strerror(err));
-#endif
-}
-
 /*
  * This is the PAM conversation function (a callback).
  * PAM repeatedly calls this function, telling us what it needs.  We pass
@@ -530,7 +518,7 @@ do_pamd(char *policy)
   static const struct pam_conv conv = { pamd_conv, NULL };
 
   if ((e = pam_start(policy, NULL, &conv, &pamh)) != PAM_SUCCESS) {
-	log_msg((LOG_ERROR_LEVEL, "pam_start() failed: %s", pam_strerr(pamh, e)));
+	log_msg((LOG_ERROR_LEVEL, "pam_start() failed: %s", PAM_STRERROR(pamh, e)));
 	return(-1);
   }
   log_msg((LOG_TRACE_LEVEL, "pam_start ok"));
@@ -609,7 +597,7 @@ do_pamd(char *policy)
   case PAM_USER_UNKNOWN:
   case PAM_MAXTRIES:
   default:
-	log_msg((LOG_ERROR_LEVEL, "do_pamd: failed: %s", pam_strerr(pamh, e)));
+	log_msg((LOG_ERROR_LEVEL, "do_pamd: failed: %s", PAM_STRERROR(pamh, e)));
 	break;
   }
 
@@ -738,11 +726,11 @@ pamd_server(in_port_t port, char *policy)
 	  close(0);
 	  if ((n = dup(cd)) == -1) {
 #endif
-	if (dup2(cd, 0) == -1) {
+	  if ((n = dup2(cd, 0)) == -1) {
 		log_err((LOG_ERROR_LEVEL, "dup"));
 		return(-1);
 	  }
-	  log_msg((LOG_TRACE_LEVEL, "pamd_server: dup(%d)=%d", cd, n));
+	  log_msg((LOG_TRACE_LEVEL, "pamd_server: dup2(%d)=%d", cd, n));
 	  if ((fp_in = fdopen(0, "r")) == NULL) {
 		log_err((LOG_ERROR_LEVEL, "Cannot open fp_in"));
 		return(-1);
@@ -752,11 +740,11 @@ pamd_server(in_port_t port, char *policy)
 	  close(1);
 	  if ((n = dup(cd)) == -1) {
 #endif
-	if (dup2(cd, 1) == -1) {
+	  if ((n = dup2(cd, 1)) == -1) {
 		log_err((LOG_ERROR_LEVEL, "dup"));
 		return(-1);
 	  }
-	  log_msg((LOG_TRACE_LEVEL, "pamd_server: dup(%d)=%d", cd, n));
+	  log_msg((LOG_TRACE_LEVEL, "pamd_server: dup2(%d)=%d", cd, n));
 	  fd_out = 1;
 	  close(cd);
 	}
@@ -991,3 +979,42 @@ main(int argc, char **argv)
 
   exit(0);
 }
+
+#else
+
+int
+main(int argc, char **argv)
+{
+  char *errmsg;
+  DACS_app_type app_type;
+  Kwv *kwv;
+
+  if (getenv("REMOTE_USER") != NULL) {
+	http_flag = 1;
+	emit_plain_header(stdout);
+	app_type = DACS_LOCAL_SERVICE;
+  }
+  else if (isatty(2))
+	app_type = DACS_UTILITY;
+  else
+	app_type = DACS_SERVER;
+
+  errmsg = NULL;
+  if (dacs_init(app_type, &argc, &argv, &kwv, &errmsg) == -1) {
+  fail:
+	if (errmsg != NULL)
+	  log_msg((LOG_ERROR_LEVEL, "%s", errmsg));
+
+	if (http_flag)
+	  emit_plain_trailer(stdout);
+
+	exit(1);
+  }
+
+  errmsg = "PAM support is unavailable.";
+  goto fail;
+  /*NOTREACHED*/
+
+  exit(1);
+}
+#endif
