@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2013
+ * Copyright (c) 2003-2016
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -12,16 +12,16 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2013\n\
+"Copyright (c) 2003-2016\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: simple_api.c 2642 2013-02-27 19:48:45Z brachman $";
+  "$Id: simple_api.c 2876 2016-05-16 23:00:00Z brachman $";
 #endif
 
-#include <pwd.h>
-
 #include "dacs_api.h"
 
+#include <pwd.h>
+
 static char *log_module_name = "simple_api";
 
 extern Kwv_vartab conf_vartab[];
@@ -127,6 +127,15 @@ get_conf(char **host)
   return(0);
 }
 
+/*
+ * HOST should be a FQDN or NULL.  If it is NULL, we really would like
+ * gethostname(2) to give us this system's (canonical) FQDN, but if it doesn't
+ * we can try a few heuristics to try to figure out a usable FQDN.
+ *
+ * Unfortunately, gethostbyname(3) is on the road to deprecation, yet
+ * there is no good substitute that will return a given host's aliases
+ * in the way that gethostbyname() does, imperfect as that function may be.
+ */
 int
 get_conf_from_host(char *host, char **hname, char **fname, char **jname,
 				   char **fdomain)
@@ -144,13 +153,13 @@ get_conf_from_host(char *host, char **hname, char **fname, char **jname,
   else
 	h = strdup(host);
 
-  if ((p = index(h, '.')) != NULL) {
+  if ((p = strchr(h, (int) '.')) != NULL) {
 	*p++ = '\0';
 	d = p;
   }
   else {
-	int i;
-	struct hostent *he;
+	int gai_code;
+	struct addrinfo ai_hints, *ai;
 
 	if (host != NULL) {
 	  /* This must be a FQDN. */
@@ -158,25 +167,25 @@ get_conf_from_host(char *host, char **hname, char **fname, char **jname,
 	}
 
 	/*
-	 * Since gethostname(3) didn't return a FQDN, look at its aliases.
+	 * Since gethostname(3) didn't return a FQDN, let's try something else.
 	 * We used to try getdomainname(3) but it will return a bogus string
 	 * (e.g., "(none)") instead of an indication that no domain name has
-	 * be configured.
+	 * be configured.  And gethostbyname(3) should be avoided.
 	 * If nothing is found, use a default string.
 	 */
+	memset(&ai_hints, 0, sizeof(ai_hints));
+	ai_hints.ai_family = AF_INET;
+	ai_hints.ai_socktype = SOCK_STREAM;
+	ai_hints.ai_protocol = IPPROTO_TCP;
+	ai_hints.ai_flags = AI_CANONNAME;
+
 	p = NULL;
-	if ((he = gethostbyname(hostname)) != NULL) {
-	  if (he->h_name == NULL || (p = index(he->h_name, '.')) == NULL) {
-		for (i = 0; he->h_aliases[i] != NULL; i++) {
-		  if ((p = index(he->h_aliases[i], '.')) != NULL) {
-			hostname = strdup(he->h_aliases[i]);
-			break;
-		  }
-		}
-	  }
-	  else if (he->h_name != NULL)
-		hostname = strdup(he->h_name);
-	}
+	ai = NULL;
+	gai_code = getaddrinfo(hostname, NULL, &ai_hints, &ai);
+	if (gai_code == 0 && ai->ai_canonname != NULL
+		&& (p = strchr(ai->ai_canonname, (int) '.')) != NULL)
+	  hostname = strdup(ai->ai_canonname);
+	freeaddrinfo(ai);
 
 	if (p == NULL) {
 	  h = hostname;
@@ -184,7 +193,7 @@ get_conf_from_host(char *host, char **hname, char **fname, char **jname,
 	}
 	else {
 	  h = strdup(hostname);
-	  p = index(h, '.');
+	  p = strchr(h, (int) '.');
 	  *p++ = '\0';
 	  d = p;
 	}
@@ -1013,10 +1022,9 @@ check_object_access(char *rules, char *groups, char *idents,
 					char *object, Kwv *kwv_conf,
 					Kwv *kwv_dacs, Acs_result *result, char **constraint)
 {
-  int st;
+  int gai_code, st;
   char *errmsg, *host, *object_url, *p, *port, *r, *str;
-  struct hostent *he;
-  struct in_addr addr;
+  struct addrinfo ai_hints, *ai;
   Acl_file *best;
   Acs_environment *env;
   Credentials *cr, *credentials;
@@ -1068,15 +1076,27 @@ check_object_access(char *rules, char *groups, char *idents,
   kwv_add_str(kwv_dacs, str);
   putenv(str);
 
-  if ((he = gethostbyname(host)) != NULL
-	  && he->h_addr_list != NULL && he->h_addr_list[0] != NULL) {
-	addr = *(struct in_addr *) he->h_addr_list[0];
-	str = ds_xprintf("SERVER_ADDR=%s", inet_ntoa(addr));
+  memset(&ai_hints, 0, sizeof(ai_hints));
+  ai_hints.ai_family = AF_INET;
+  ai_hints.ai_socktype = SOCK_STREAM;
+  ai_hints.ai_protocol = IPPROTO_TCP;
+  ai_hints.ai_flags = AI_CANONNAME;
+
+  ai = NULL;
+  gai_code = getaddrinfo(host, NULL, &ai_hints, &ai);
+  if (gai_code != 0)
+	log_msg((LOG_WARN_LEVEL, "Can't determine SERVER_ADDR for \"%s\"", host));
+  else {
+    struct in_addr sin_addr;
+    struct sockaddr_in *sai;
+
+    sai = (struct sockaddr_in *) ai->ai_addr;
+    sin_addr = sai->sin_addr;
+    str = ds_xprintf("SERVER_ADDR=%s", inet_ntoa(sin_addr));
 	kwv_add_str(kwv_dacs, str);
 	putenv(str);
+	freeaddrinfo(ai);
   }
-  else
-	log_msg((LOG_WARN_LEVEL, "Can't determine SERVER_ADDR for \"%s\"", host));
 
   if (uri->port_given != NULL)
 	port = ds_xprintf("%d", uri->port);
