@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2017
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -12,10 +12,10 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2012\n\
+"Copyright (c) 2003-2017\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: sts.c 2586 2012-03-15 16:21:40Z brachman $";
+  "$Id: sts.c 2971 2017-07-06 18:00:47Z brachman $";
 #endif
 
 #include "icx.h"
@@ -471,7 +471,7 @@ decrypt_ppid(xmlChar *cv1, xmlChar *cv2, char *privatekeyblock,
    * In Rijndael, the IV is always the same size as the key, but in AES the
    * IV is always 16 bytes.
    */
-  if (crypto_cipher_iv_length("aes-256-cbc", &iv_len) == -1)
+  if (crypto_lookup_cipher_iv_length("aes-256-cbc", &iv_len) == -1)
 	return(NULL);
   log_msg((LOG_TRACE_LEVEL, "IV length is %u bytes", iv_len));
 
