@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2016
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -39,8 +39,12 @@
 
 /*
  * Generic virtual filestore (VFS) layer
- * This is an API to perform operations on stored objects (which may be local
- * or remote) and a command line utility to access the API.
+ * This is:
+ * 1) an API to perform operations on stored objects (which may be local
+ *    or remote);
+ * 2) a command line utility to access the API;
+ * 3) an interactive utility to access the API; and
+ * 4) web services to access the API.
  *
  * This generic layer together with a collection of store "modules" implements
  * a simple virtual filestore that supports basic operations on objects.
@@ -58,10 +62,10 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2012\n\
+"Copyright (c) 2003-2016\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: vfs.c 2542 2012-01-11 19:40:13Z brachman $";
+  "$Id: vfs.c 2902 2016-07-05 14:45:35Z brachman $";
 #endif
 
 #include "local.h"
@@ -388,6 +392,33 @@ vfs_lookup_op(char *opname)
   return(VFS_UNKNOWN);
 }
 
+/*
+ * Return a list of all defined item type definitions, or NULL if an error
+ * occurs.
+ */
+Dsvec *
+vfs_list_item_types(Dsvec *odsv)
+{
+  int i;
+  Dsvec *dsv;
+  Kwv_pair *v;
+
+  if (odsv != NULL)
+	dsv = odsv;
+  else
+	dsv = dsvec_init(NULL, sizeof(char *));
+
+  if ((v = conf_var(CONF_VFS)) == NULL)
+	return(NULL);
+
+  while (v != NULL) {
+	dsvec_add_ptr(dsv, strdup(v->val));
+	v = v->next;
+  }
+
+  return(dsv);
+}
+
 /*
  * Search the configuration for a VFS directive for ITEM_TYPE.
  * A VFS directive for the item type "default" establishes a default
@@ -990,14 +1021,17 @@ vfss_get(Vfs_handle *handle, char *key, char **errmsg)
 {
   char *ptr;
   size_t length;
+  ssize_t nwritten;
 
   if (vfs_get(handle, key, (void *) &ptr, &length) == -1)
 	return(-1);
-  else {
-	if (app_type == DACS_WEB_SERVICE)
-	  printf("\n");
-	printf("%s%s", ptr, key != NULL ? "\n" : "");
-  }
+
+  if (app_type == DACS_WEB_SERVICE)
+	nwritten = write(1, "\n", 1);
+
+  nwritten = write(1, ptr, length);
+  if (key != NULL)
+	nwritten = write(1, "\n", 1);
 
   return(0);
 }
@@ -1033,7 +1067,7 @@ do_update(Vfs_handle *handle, char *key, char **ermsg)
   fclose(fp);
   memzap(ptr, length);
 
-  if (edit_file(tempfile) != 0) {
+  if (edit_file(tempfile) != 1) {
 	fprintf(stderr, "do_update: aborted\n");
 	if (unlink(tempfile) == -1)
 	  fprintf(stderr, "do_update: error unlinking tempfile \"%s\": %s\n",
@@ -1107,7 +1141,8 @@ vfss_exists(Vfs_handle *handle, char *key, char **ermsg)
 
   if ((st = vfs_exists(handle, key)) == -1)
 	return(-1);
-  else if (st == 1) {
+
+  if (st == 1) {
 	if (app_type == DACS_WEB_SERVICE)
 	  printf("\nyes\n");
   }
@@ -1267,7 +1302,7 @@ vfss_list(Vfs_handle *handle,  int (*is_valid)(char *),
 }
 
 /*
- * A VFS web service
+ * Perform a VFS web service.
  */
 static int
 vfss(Kwv *kwv, char **errmsg)
@@ -1279,12 +1314,14 @@ vfss(Kwv *kwv, char **errmsg)
   Vfs_handle *handle;
   Mkargv conf = { 0, 0, NULL, NULL, NULL };
 
+  /* Look up the arguments. */
   item_type = kwv_lookup_value(kwv, "ITEM_TYPE");
   jurisdiction = kwv_lookup_value(kwv, "DACS_JURISDICTION");
   request = kwv_lookup_value(kwv, "REQUEST");
   value = kwv_lookup_value(kwv, "VALUE");
   version = kwv_lookup_value(kwv, "DACS_VERSION");
   sep = kwv_lookup_value(kwv, "FIELD_SEP");
+
   vfs_config = vfs_conf(NULL);
   vfs_config->null_flag = (p = kwv_lookup_value(kwv, "NULL_FLAG")) != NULL
 	&& streq(p, "1");
@@ -1294,6 +1331,7 @@ vfss(Kwv *kwv, char **errmsg)
 	&& streq(p, "1");
   vfs_config->append_flag = (p = kwv_lookup_value(kwv, "APPEND_FLAG")) != NULL
 	&& streq(p, "1");
+
   if ((p = kwv_lookup_value(kwv, "MODE")) != NULL) {
 	if (strnum_b(p, STRNUM_MODE_T, 8, &vfs_config->mode, NULL) == -1)
 	  vfs_config->mode = 0;
@@ -1318,6 +1356,7 @@ vfss(Kwv *kwv, char **errmsg)
 	st = -1;
 	goto fail;
   }
+
   if (!vfs_is_valid_item_type(item_type)) {
 	log_msg((LOG_ERROR_LEVEL, "Invalid item_type: \"%s\"", item_type));
 	if (errmsg != NULL)
@@ -1484,7 +1523,7 @@ static void
 show_usage(void)
 {
 
-  if (quiet_flag)
+  if (dacs_quiet_flag)
 	exit(1);
 
   fprintf(stderr, "Usage:\n");
@@ -1558,21 +1597,23 @@ dacsvfs_main(int argc, char **argv, int do_init, void *main_out)
   crypt_keys_free(keys);
 
   if (app_type == DACS_WEB_SERVICE) {
+	/* Perform one web service request and then exit. */
 	errmsg = NULL;
-	if (vfss(kwv, &errmsg) == -1) {
+	if (vfss(kwv, &errmsg) == -1)
 	  goto fail;
-	}
 
 	return(0);
   }
 
+  /* Everything that follows is only done when this is invoked as a command. */
+
   if (argv[1] == NULL) {
 	fprintf(stderr, "Missing item-type or vfs_uri argument\n");
 	show_usage();
 	return(-1);
   }
 
-  /* A special case */
+  /* A special case - print enabled store names, then exit. */
   if (streq(argv[1], "enabled")) {
 	vfss_list_enabled();
 	return(0);
@@ -1723,7 +1764,7 @@ dacsvfs_main(int argc, char **argv, int do_init, void *main_out)
 	  return(-1);
 	}
 
-	if (st == -1 && !quiet_flag) {
+	if (st == -1) {
 	  if (handle != NULL && handle->error_msg != NULL)
 		fprintf(stderr, "%s\n", handle->error_msg);
 	  if (errmsg != NULL)
