@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2016
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -44,10 +44,10 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2012\n\
+"Copyright (c) 2003-2016\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: vfs_fs.c 2594 2012-10-19 17:28:49Z brachman $";
+  "$Id: vfs_fs.c 2913 2016-10-18 19:55:39Z brachman $";
 #endif
 
 #include "local.h"
@@ -231,6 +231,15 @@ fs_control(Vfs_handle *handle, Vfs_control_op op, va_list ap)
 
 	break;
 
+  case VFS_SET_FIELD_SEP:
+	/* Nothing to do... */
+	break;
+
+  case VFS_GET_FIELD_SEP:
+	return(-1);
+	/*NOTREACHED*/
+	break;
+
   default:
     log_msg((LOG_ERROR_LEVEL, "Invalid control request: %d", op));
     return(-1);
@@ -255,6 +264,12 @@ getsize(char *path, size_t *length)
   return(0);
 }
 
+/*
+ * This assumes that a regular file (not a stream) is being read.
+ * That was adequate for the original functionality that was needed,
+ * such as slurping up a password file, but is obviously lacking for other
+ * purposes.
+ */
 static int
 fs_get(Vfs_handle *handle, char *key, void **buffer, size_t *length)
 {
@@ -305,6 +320,7 @@ fs_get(Vfs_handle *handle, char *key, void **buffer, size_t *length)
 	  handle->error_msg = strdup("fread did not return correct byte count");
 	st = -1;
   }
+
   if (ferror(fp)) {
 	if (handle->error_msg == NULL) {
 	  handle->error_num = errno;
@@ -312,6 +328,7 @@ fs_get(Vfs_handle *handle, char *key, void **buffer, size_t *length)
 	}
 	st = -1;
   }
+
   if (fclose(fp) == EOF) {
 	if (handle->error_msg == NULL) {
 	  handle->error_num = errno;
