@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2015
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -10,10 +10,10 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2012\n\
+"Copyright (c) 2003-2015\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: vfs_ndbm.c 2594 2012-10-19 17:28:49Z brachman $";
+  "$Id: vfs_ndbm.c 2829 2015-08-10 22:32:35Z brachman $";
 #endif
 
 #include "local.h"
@@ -24,8 +24,12 @@ static const char revid[] =
 /*
  * If we're using gdbm, the gdbm compatibility file needs to be used here.
  */
+#ifndef __BSD_VISIBLE
+#define __BSD_VISIBLE
+#endif
 #include <ndbm.h>
 
+#ifdef NOTDEF
 /* We need to provide function prototypes. */
 extern DBM *dbm_open(const char *file, int flags, int mode);
 extern void dbm_close(DBM *db);
@@ -38,6 +42,7 @@ extern int dbm_store(DBM *db, datum key, datum data, int flags);
 extern int dbm_error(DBM *db);
 #endif
 extern int dbm_dirfno(DBM *db);
+#endif
 
 static const char *log_module_name = "vfs_ndbm";
 
@@ -148,6 +153,15 @@ vfs_ndbm_control(Vfs_handle *handle, Vfs_control_op op, va_list ap)
 	*pptr = ds_xprintf("file:%s", h->dbmfile);
 	break;
 
+  case VFS_SET_FIELD_SEP:
+	/* Nothing to do... */
+	break;
+
+  case VFS_GET_FIELD_SEP:
+	return(-1);
+	/*NOTREACHED*/
+	break;
+
   default:
 	log_msg((LOG_ERROR_LEVEL, "Invalid control request: %d", op));
 	return(-1);
