@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2014
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -43,10 +43,10 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2012\n\
+"Copyright (c) 2003-2014\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: xml.c 2592 2012-03-28 22:37:50Z brachman $";
+  "$Id: xml.c 2707 2014-09-04 22:57:04Z brachman $";
 #endif
 
 #ifdef DSSLIB
@@ -79,6 +79,8 @@ parse_xml_init(char *name, XML_Parser p)
   parse_xml_error.mesg = NULL;
   parse_xml_error.line = -1;
   parse_xml_error.pos = -1;
+  parse_xml_error.handler = NULL;
+
   parse_xml_index = -1;
   parse_xml_parser = p;
   parse_xml_complete = 0;
@@ -106,6 +108,13 @@ parse_xml_is_complete(void)
   return(parse_xml_complete);
 }
 
+void
+parse_xml_set_error_handler(void (*handler)(Parse_xml_error *))
+{
+
+  parse_xml_error.handler = handler;
+}
+
 void
 parse_xml_set_error(char *mesg)
 {
@@ -126,6 +135,12 @@ parse_xml_set_error(char *mesg)
 
   parse_xml_error.line = XML_GetCurrentLineNumber(parse_xml_parser);
   parse_xml_error.pos = XML_GetCurrentColumnNumber(parse_xml_parser);
+
+  /*
+   * If a user-specified error handler function has been configured, call it.
+   */
+  if (parse_xml_error.handler != NULL)
+	parse_xml_error.handler(&parse_xml_error);
 }
 
 int
